/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <ktextedit.h>

#include "authconfigdlg.h"
#include "authenticate.h"
#include "mainwidget.h"

//	constructor
//	===========

AuthConfigDlg::AuthConfigDlg( CPtr client, QWidget* parent, const char* name )
: KDialogBase( Plain, i18n( "Authentication configuration" ), Ok | Cancel, Ok, parent, name ),
  m_authReqd( client->authenticate->m_authReqd ),
  m_autoAuthenticate( client->authenticate->m_autoAuthenticate ),
  m_authScript( client->authenticate->m_authScript ),
  m_clearIconSet( KGlobal::iconLoader( )->loadIconSet( "locationbar_erase", KIcon::Small )),
  m_client( client ),
  m_sameClient( client->authenticate->m_sameClient ),
  m_sameClientName( client->authenticate->m_sameClientName )
{	AuthVariable* authVar;
	QPtrListIterator<AuthVariable> it( client->authenticate->m_authVariables );

    while( (authVar = it.current( )) != NULL )
	{	m_authVariables.append( authVar );
		++it;
	}

	createLayout( );
	slotSameLayoutClicked( );
	enableStuff( m_authReqd );
	enableSameBox( );
	resize( 600, 600 );
}

//	createLayout
//	============

void AuthConfigDlg::createLayout( )
{	m_layout = new QGridLayout( plainPage( ));
	byte row = 0;

	m_layout->setRowStretch( row++, 1 );

	QVBoxLayout* vbox = new QVBoxLayout( );
	m_authReqdBox     = new QCheckBox( i18n( "Authentication required"  ), plainPage( ), "auth reqd box" );
	m_autoBox         = new QCheckBox( i18n( "Automatic authentication" ), plainPage( ), "auto auth box" );
	m_authReqdBox->setChecked( m_authReqd );
	m_autoBox->setChecked( m_autoAuthenticate );
	vbox->addWidget( m_authReqdBox );
	vbox->addWidget( m_autoBox );
	m_layout->addMultiCellLayout( vbox, row, row, 1, 11, Qt::AlignCenter );
	row++;
	connect( m_authReqdBox, SIGNAL( clicked( )), SLOT( slotAuthReqdClicked( )));
	m_layout->setRowStretch( row++, 1 );

	m_layout->addMultiCellLayout( createSameLayout( ), row, row, 1, 11, Qt::AlignCenter );
	row++;
	m_layout->setRowStretch( row++, 1 );

	m_layout->addMultiCellWidget( new QLabel( i18n( "Variable name"  ), plainPage( ), "name label"  ), row, row, 1, 2, Qt::AlignCenter );
	m_layout->addMultiCellWidget( new QLabel( i18n( "Variable value" ), plainPage( ), "value label" ), row, row, 4, 5, Qt::AlignCenter );
	m_layout->setRowStretch( row++, 1 );

	AuthVariable* authVar;

	for( authVar = m_authVariables.first( ); authVar != NULL; row++, authVar = m_authVariables.next( ))
	{	createOneRow( row );

		m_nameEdits.last( )->setText( authVar->name( ));
		m_secureBoxes.last( )->setChecked( authVar->secure( ));
		m_showValues.last( )->setEnabled( authVar->secure( ));
		m_valueEdits.last( )->setText( authVar->value( ));

		if( authVar->secure( ))
			m_valueEdits.last( )->setEchoMode( QLineEdit::Password );
	}

	createOneRow( row );
	m_secureBoxes.last( )->setChecked( false );
	m_showValues.last( )->setEnabled( false );

	m_more = new QPushButton( i18n( "More" ), plainPage( ), "more button" );
	connect( m_more, SIGNAL( clicked( )), SLOT( slotMoreClicked( )));
	m_layout->addWidget( m_more, row, 11 );
	row++;

	m_layout->setRowStretch( row, 1 );
	m_lastVarRow = row++;

	m_scriptEdit = new KTextEdit( m_authScript.join( "\n" ), QString::null, plainPage( ), "auth text edit" );
	m_scriptEdit->setTextFormat( Qt::PlainText );

	m_scriptLabel = new QLabel( i18n( "Authentication script" ), plainPage( ), "script label" );
	m_layout->addMultiCellWidget( m_scriptLabel, row, row, 1, 11 );
	m_layout->setRowStretch( row++, 1 );
	m_layout->addMultiCellWidget( m_scriptEdit, row, row, 1, 11 );
	m_layout->setRowStretch( row++, 40 );
	m_layout->setRowStretch( row++, 1 );

	m_layout->setColStretch(  0,  1 );
	m_layout->setColStretch(  2, 10 );
	m_layout->setColStretch(  3,  1 );
	m_layout->setColStretch(  5, 10 );
	m_layout->setColStretch(  6,  1 );
	m_layout->setColStretch(  8,  1 );
	m_layout->setColStretch( 10,  1 );
	m_layout->setColStretch( 12,  1 );
}

//	createOneRow
//	============

void AuthConfigDlg::createOneRow( const byte row )
{	ClearButton* clearNameButton  = new ClearButton( m_clearIconSet,  plainPage( ), "clear name button"  );
	ClearButton* clearValueButton = new ClearButton( m_clearIconSet,  plainPage( ), "clear value button" );
	KLineEdit*   nameEdit         = new KLineEdit( QString::null,     plainPage( ), "name edit"   );
	KLineEdit*   valueEdit        = new KLineEdit( QString::null,     plainPage( ), "value edit"  );
	SecureBox*   secureBox        = new SecureBox( i18n( "Secure?" ), plainPage( ), "secure box"  );
	ShowButton*  showButton       = new ShowButton( i18n( "Show" ),   plainPage( ), "show button" );

	m_clearNames.append( clearNameButton );
	m_clearValues.append( clearValueButton );
	m_nameEdits.append( nameEdit );
	m_secureBoxes.append( secureBox );
	m_showValues.append( showButton );
	m_valueEdits.append( valueEdit );

	m_layout->addWidget( clearNameButton,  row, 1 );
	m_layout->addWidget( nameEdit,         row, 2 );
	m_layout->addWidget( clearValueButton, row, 4 );
	m_layout->addWidget( valueEdit,        row, 5 );
	m_layout->addWidget( secureBox,        row, 7 );
	m_layout->addWidget( showButton,       row, 9 );

	connect( clearNameButton,
			 SIGNAL( clicked( ClearButton* const )),
			 SLOT( slotClearNameClicked( ClearButton* const )));
	connect( clearValueButton,
			 SIGNAL( clicked( ClearButton* const )),
			 SLOT( slotClearValueClicked( ClearButton* const )));
	connect( secureBox,
			 SIGNAL( clicked( SecureBox* const )),
			 SLOT( slotSecureClicked( SecureBox* const )));
	connect( showButton,
			 SIGNAL( clicked( ShowButton* const )),
			 SLOT( slotShowClicked( ShowButton* const )));
}

//	createSameLayout
//	================

QGridLayout* AuthConfigDlg::createSameLayout( )
{	QPtrList<Client> clients = MainWidget::self( )->getClients( );
	QGridLayout* sameLayout  = new QGridLayout( 1 );

	m_sameBox   = new QCheckBox( i18n( "Same script as BOINC client ->" ), plainPage( ), "same script box" );
	m_sameCombo = new QComboBox( plainPage( ), "same combo" );

	m_sameBox->setChecked( m_sameClient );
	CPtr client;
	uint index;

	for( index = 0, client = clients.first( ); client != NULL; client = clients.next( ))
		if( client != m_client )
		{	m_sameCombo->insertItem( client->clientName, index );

			if( client->clientName == m_sameClientName )
				m_sameCombo->setCurrentItem( index );

			index++;
		}

	sameLayout->addWidget( m_sameBox,   0, 1 );
	sameLayout->addWidget( m_sameCombo, 0, 3);

	sameLayout->setColStretch(  0,  10 );
	sameLayout->setColStretch(  2,   1 );
	sameLayout->setColStretch(  4,  10 );

	connect( m_sameBox, SIGNAL( clicked( )), SLOT( slotSameLayoutClicked( )));
	return sameLayout;
}


//	enableSameBox
//	=============

void AuthConfigDlg::enableSameBox( )
{	if( !m_authReqdBox->isChecked( ))
		m_sameCombo->setEnabled( false );
	else
		m_sameCombo->setEnabled( m_sameBox->isChecked( ));
}

//	enableStuff
//	===========

void AuthConfigDlg:: enableStuff( const bool enable )
{	enableScriptStuff( enable && !m_sameBox->isChecked( ));

	m_autoBox->setEnabled( enable );
	m_sameBox->setEnabled( enable );
}

//	enableScriptStuff
//	=================

void AuthConfigDlg::enableScriptStuff( const bool enable )
{	m_more->setEnabled( enable );
	m_scriptEdit->setEnabled( enable );

	for( QWidget* widget = m_clearNames.first( ); widget != NULL; widget = m_clearNames.next( ))
		widget->setEnabled( enable );

	for( QWidget* widget = m_clearValues.first( ); widget != NULL; widget = m_clearValues.next( ))
		widget->setEnabled( enable );

	for( QWidget* widget = m_nameEdits.first( ); widget != NULL; widget = m_nameEdits.next( ))
		widget->setEnabled( enable );

	for( QWidget* widget = m_secureBoxes.first( ); widget != NULL; widget = m_secureBoxes.next( ))
		widget->setEnabled( enable );

	for( QWidget* widget = m_showValues.first( ); widget != NULL; widget = m_showValues.next( ))
		widget->setEnabled( enable );

	for( QWidget* widget = m_valueEdits.first( ); widget != NULL; widget = m_valueEdits.next( ))
		widget->setEnabled( enable );
}

//	slotAuthReqdClicked
//	===================

void AuthConfigDlg::slotAuthReqdClicked( )
{	enableStuff( m_authReqdBox->isChecked( ));
	enableSameBox( );
}

//	slotClearNameClicked
//	====================

void AuthConfigDlg::slotClearNameClicked( ClearButton* const clearButton )
{	ushort i;
	ClearButton* listButton;

	for( i = 0, listButton = m_clearNames.first( ); listButton != NULL; i++, listButton = m_clearNames.next( ))
		if( listButton == clearButton )
		{	m_nameEdits.at( i )->clear( );
			return;
		}

	Q_ASSERT( false );
}

//	slotClearValueClicked
//	=====================

void AuthConfigDlg::slotClearValueClicked( ClearButton* const clearButton )
{	ushort i;
	ClearButton* listButton;

	for( i = 0, listButton = m_clearValues.first( ); listButton != NULL; i++, listButton = m_clearValues.next( ))
		if( listButton == clearButton )
		{	m_valueEdits.at( i )->clear( );
			return;
		}

	Q_ASSERT( false );
}

//	slotMoreClicked
//	===============

void AuthConfigDlg::slotMoreClicked( )
{	m_layout->remove( m_more );
	m_layout->remove( m_scriptEdit );
	m_layout->remove( m_scriptLabel );
	m_layout->setRowStretch( m_lastVarRow, 0 );

	createOneRow( m_lastVarRow );

	m_secureBoxes.last( )->setChecked( false );
	m_showValues.last( )->setEnabled( false );

	m_clearNames.last( )->show( );
	m_clearValues.last( )->show( );
	m_nameEdits.last( )->show( );
	m_secureBoxes.last( )->show( );
	m_showValues.last( )->show( );
	m_valueEdits.last( )->show( );

	m_layout->addWidget( m_more, m_lastVarRow, 11 );
	m_lastVarRow++;

	m_layout->setRowStretch( m_lastVarRow, 1 );
	byte row = m_lastVarRow + 1;

	m_layout->addMultiCellWidget( m_scriptLabel, row, row, 1, 9 );
	m_layout->setRowStretch( row++, 1 );
	m_layout->addMultiCellWidget( m_scriptEdit, row, row, 1, 9 );
	m_layout->setRowStretch( row++, 10 );
	m_layout->setRowStretch( row++, 1 );
}

//	slotOk
//	======

void AuthConfigDlg::slotOk( )
{	m_authReqd         = m_authReqdBox->isChecked( );
	m_authScript       = QStringList::split( '\n', m_scriptEdit->text( ));
	m_autoAuthenticate = m_autoBox->isChecked( );
	m_sameClient       = m_sameBox->isChecked( );
	m_sameClientName   = m_sameCombo->currentText( );

	KLineEdit* nameEdit  = m_nameEdits.first( );
	QCheckBox* secureBox = m_secureBoxes.first( );
	KLineEdit* valueEdit = m_valueEdits.first( );

	m_authVariables.clear( );

	while( true )
	{	if( nameEdit == NULL )
		{	Q_ASSERT( secureBox == NULL );
			Q_ASSERT( valueEdit == NULL );
			break;
		}

		if( !nameEdit->text( ).isEmpty( ))
			m_authVariables.append( new AuthVariable( nameEdit->text( ),
													  secureBox->isChecked( ),
													  valueEdit->text( )));
		nameEdit  = m_nameEdits.next( );
		secureBox = m_secureBoxes.next( );
		valueEdit = m_valueEdits.next( );
	}

	KDialogBase::slotOk( );
}

//	slotSameLayoutClicked
//	=====================

void AuthConfigDlg::slotSameLayoutClicked( )
{	enableScriptStuff( !m_sameBox->isChecked( ));
	enableSameBox( );
}

//	slotSecureClicked
//	=================

void AuthConfigDlg::slotSecureClicked( SecureBox* const selectBox )
{	ushort i;
	SecureBox* listBox;

	for( i = 0, listBox = m_secureBoxes.first( ); listBox != NULL; i++, listBox = m_secureBoxes.next( ))
		if( listBox == selectBox )
			break;

	if( listBox == NULL )
	{	Q_ASSERT( false );
		return;
	}

	if( selectBox->isChecked( ))
	{	m_valueEdits.at( i )->setEchoMode( QLineEdit::Password );
		m_showValues.at( i )->setEnabled( true );
		m_showValues.at( i )->text( "Show" );
	}
	else
	{	m_valueEdits.at( i )->setEchoMode( QLineEdit::Normal );
		m_showValues.at( i )->setEnabled( false );
		m_showValues.at( i )->text( "Hide" );
	}
}

//	slotShowClicked
//	===============

void AuthConfigDlg::slotShowClicked( ShowButton* const showButton )
{	ushort i;
	ShowButton* listButton;

	for( i = 0, listButton = m_showValues.first( ); listButton != NULL; i++, listButton = m_showValues.next( ))
		if( listButton == showButton )
			break;

	if( listButton == NULL )
	{	Q_ASSERT( false );
		return;
	}

	if( showButton->text( ) == "Hide" )
	{	m_valueEdits.at( i )->setEchoMode( QLineEdit::Password );
		showButton->text( "Show" );
	}
	else
	{	m_valueEdits.at( i )->setEchoMode( QLineEdit::Normal );
		showButton->text( "Hide" );
	}
}

