/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _AUTHCONFIGDLG_H_
#define _AUTHCONFIGDLG_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qcheckbox.h>
#include <qiconset.h>
#include <qptrlist.h>
#include <qstringlist.h>

#include <kdialogbase.h>
#include <klineedit.h>

#include "authvariable.h"
#include "clearbutton.h"
#include "global.h"
#include "securebox.h"
#include "showbutton.h"

//	forward class definitions
//	=========================

class KTextEdit;
class QCheckBox;
class QComboBox;
class QGridLayout;
class QLabel;
class QPushButton;
class TransfersControlView;

//	AuthConfigDlg class definition
//	==============================

class AuthConfigDlg : public KDialogBase
{	friend class Authenticate;

	Q_OBJECT

	public:
		AuthConfigDlg( CPtr client, QWidget* parent, const char* name );

	private slots:
		void slotAuthReqdClicked( );
		void slotClearNameClicked( ClearButton* const clearButton );
		void slotClearValueClicked( ClearButton* const clearButton );
		void slotMoreClicked( );
		void slotSameLayoutClicked( );
		void slotSecureClicked( SecureBox* const selectBox );
		void slotShowClicked( ShowButton* const showButton );

	private:
		void         createLayout( );
		void         createOneRow( const byte row );
		QGridLayout* createSameLayout( );
		void         enableSameBox( );
		void         enableStuff( const bool enable );
		void         enableScriptStuff( const bool enable );
		void         slotOk( );

		bool                   m_authReqd;
		QCheckBox*             m_authReqdBox;
		bool                   m_autoAuthenticate;
		QCheckBox*             m_autoBox;
		QStringList            m_authScript;
		QPtrList<AuthVariable> m_authVariables;
		QIconSet               m_clearIconSet;
		QPtrList<ClearButton>  m_clearNames;
		QPtrList<ClearButton>  m_clearValues;
		CPtr                   m_client;
		byte                   m_lastVarRow;
		QGridLayout*           m_layout;
		QPushButton*           m_more;
		QPtrList<KLineEdit>    m_nameEdits;
		QCheckBox*             m_sameBox;
		QComboBox*             m_sameCombo;
		bool                   m_sameClient;
		QString                m_sameClientName;
		KTextEdit*             m_scriptEdit;
		QLabel*                m_scriptLabel;
		QPtrList<SecureBox>    m_secureBoxes;
		QPtrList<ShowButton>   m_showValues;
		QPtrList<KLineEdit>    m_valueEdits;
};

#endif	// _AUTHCONFIGDLG_H_
