/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>

#include <qtimer.h>
#include <qwidget.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kprocess.h>

#include "authconfigdlg.h"
#include "authenticate.h"
#include "mainwidget.h"
#include "mainwidgetview.h"

//	constructor
//	===========

Authenticate::Authenticate( const CPtr client, Wallet* wallet, QObject* parent, const char* name )
: QObject( parent, name ),
  m_authDataIsCrap( true ),
  m_buffer( NULL ),
  m_bufferLength( 0 ),
  m_client( client ),
  m_wallet( wallet )
{	m_authVariables.setAutoDelete( true );
	readSettings( );
}

//	destructor
//	==========

Authenticate::~Authenticate( )
{	if( m_buffer != NULL )
		delete [] m_buffer;
}

//	authenticate
//	============

bool Authenticate::authenticate( )
{	if( !m_sameClient )
		m_authScriptClient = m_client;
	else
	{	QPtrList<Client> clients = MainWidget::self( )->getClients( );

		for( m_authScriptClient = clients.first( ); m_authScriptClient != NULL; m_authScriptClient = clients.next( ))
			if( m_sameClientName == m_authScriptClient->clientName )
				break;

		if( m_authScriptClient == NULL || m_authScriptClient->authenticate == NULL )
		{	KMessageBox::sorry( m_client->listView, i18n( QString( "Cannot authenticate using the same script as \"%1\", that client does not exist!" ).arg( m_sameClientName )), i18n( "Cannot authenticate" ));
			return false;
		}

		m_authScriptClient->locked = true;
	}

	m_passwdStuff = getpwuid( getuid( ));

	if( m_passwdStuff == NULL )
	{	KMessageBox::sorry( m_client->listView, i18n( "Cannot determine your logon shell" ), i18n( "Authntication failed" ));
		return false;
	}

	KProcess* process = new KProcess;
	*process << m_passwdStuff->pw_shell;

	connect( process,
			 SIGNAL( processExited( KProcess* )),
			 SLOT( slotProcessExited( KProcess* )));
	connect( process,
			 SIGNAL( receivedStdout( KProcess*, char*, int )),
			 SLOT( slotReceivedStdout( KProcess*, char*, int )));
	connect( process,
			 SIGNAL( receivedStderr( KProcess*, char*, int )),
			 SLOT( slotReceivedStderr( KProcess*, char*, int )));
	connect( process,
			 SIGNAL( wroteStdin( KProcess* )),
			 SLOT( slotWroteStdin( KProcess* )));

	if( !process->start( KProcess::NotifyOnExit, KProcess::All ))
	{	KMessageBox::sorry( m_client->listView, i18n( "Couldn't start the authentication process" ), i18n( "Authentication failed" ));
		return false;
	}

	m_authScriptIndex = 0;
	slotWroteStdin( process );

	emit( authenticationStarted( m_client ));
	emit( disableAuthButton( ));

	return true;
}

//	configure
//	=========

void Authenticate::configure( )
{	AuthConfigDlg dlg( m_client, m_client->listView, "auth config dlg" );

	if( dlg.exec( ) != QDialog::Accepted )
		return;

	for( AuthVariable* authVar = m_authVariables.first( ); authVar != NULL; authVar = m_authVariables.next( ))
	{	if( !authVar->secure( ))
			continue;

		if( m_wallet == NULL )
			m_wallet = MainWidget::self( )->openWallet( );

		if( m_wallet != NULL )
			m_wallet->removeEntry( authVar->name( ));
	}

	m_autoAuthenticate = dlg.m_autoAuthenticate;
	m_authReqd         = dlg.m_authReqd;
	m_authScript       = dlg.m_authScript;
	m_sameClient       = dlg.m_sameClient;
	m_sameClientName   = dlg.m_sameClientName;

	m_authVariables.clear( );

	for( AuthVariable* authVar = dlg.m_authVariables.first( ); authVar != NULL; authVar = dlg.m_authVariables.next( ))
		m_authVariables.append( authVar );

	saveSettings( );

	if( m_authReqd )
		emit( enableAuthButton( ));
	else
		emit( disableAuthButton( ));
}

//	createBuffer
//	============

void Authenticate::createBuffer( const uint length )
{	if( length <= m_bufferLength )
		return;

	delete [] m_buffer;

	m_bufferLength = length;
	m_buffer = new char[ m_bufferLength ];
}

//	processScriptLine
//	=================

int Authenticate::processScriptLine( )
{	if( m_authScriptIndex == m_authScriptClient->authenticate->m_authScript.count( ))
	{	m_authScriptIndex++;
		return 0;
	}

	if( m_authScriptIndex >= m_authScriptClient->authenticate->m_authScript.count( ))
		return 1;

	m_authText       = m_authScriptClient->authenticate->m_authScript[ m_authScriptIndex ] + '\n';
	m_authTextMasked = m_authScriptClient->authenticate->m_authScript[ m_authScriptIndex ] + '\n';

	for( AuthVariable* authVar = m_authScriptClient->authenticate->m_authVariables.first( ); authVar != NULL; authVar = m_authScriptClient->authenticate->m_authVariables.next())
	{	QString name = "${" + authVar->name( ) + "}";
		m_authText.replace( name, authVar->value( ));
		m_authTextMasked.replace( name, authVar->valueMasked( ));
	}

	m_authText.replace( "${HOME}", QString( m_passwdStuff->pw_dir ));
	m_authText.replace( "${HOST}", m_client->clientName );

	m_authTextMasked.replace( "${HOME}", QString( m_passwdStuff->pw_dir ));
	m_authTextMasked.replace( "${HOST}", m_client->clientName );

	m_authScriptIndex++;
	return -1;
}

//	readSettings
//	============

void Authenticate::readSettings( )
{	KConfig* config = kapp->config( );
	config->setGroup( "Authentication_" + m_client->clientName );

	m_authReqd         = config->readBoolEntry( "authReqd", false );
	m_authScript       = config->readListEntry( "authScript" );
	m_autoAuthenticate = config->readBoolEntry( "autoAuthenticate", false );
	m_sameClient       = config->readBoolEntry( "sameClient", false );
	m_sameClientName   = config->readEntry( "sameClientName" );
	QStringList list   = config->readListEntry( "authVariables" );

	for( QStringList::Iterator it = list.begin( ); it != list.end( ); ++it )
	{	AuthVariable* authVar = new AuthVariable( *it );
		m_authVariables.append( authVar );

		if( !authVar->secure( ))
			continue;

		if( m_wallet == NULL )
			m_wallet = MainWidget::self( )->openWallet( );

		if( m_wallet == NULL )
			return;

		QString value;

		if( !m_wallet->readPassword( "KBoincMgr_" + authVar->name( ) + "_" + m_client->clientName, value ))
		{	authVar->value( value );
			continue;
		}

		m_authDataIsCrap = true;
		m_errorCaption   = i18n( "Couldn't read variable value" );
		m_errorText      = i18n( QString( "Could read the value of \"%1\" for client \"%2\" from the wallet" ).arg( authVar->name( )).arg( m_client->clientName ));

		QTimer::singleShot( 0, this, SLOT( slotErrorTimeout( )));
		return;
	}

	m_authDataIsCrap = false;
}

//	saveSettings
//	============

void Authenticate::saveSettings( )
{	if( m_authDataIsCrap )
		return;

	QStringList list;

	for( AuthVariable* authVar = m_authVariables.first( ); authVar != NULL; authVar = m_authVariables.next( ))
	{	list += authVar->toString( );

		if( !authVar->secure( ))
			continue;

		if( m_wallet == NULL )
			m_wallet = MainWidget::self( )->openWallet( );

		if( m_wallet == NULL || m_wallet->writePassword( "KBoincMgr_" + authVar->name( ) + "_" + m_client->clientName, authVar->value( )))
		{	KMessageBox::error( m_client->listView, i18n( QString( "Could not save the value of the variable \"%1\" in the wallet. No other variables / authorisation script information will be saved. Your authorisation script has been corrupted!" ).arg( authVar->name( ))), i18n( "Couldn't save variable value" ));
			return;
		}
	}

	KConfig* config = kapp->config( );
	config->setGroup( "Authentication_" + m_client->clientName );
	config->writeEntry( "authReqd",         m_authReqd         );
	config->writeEntry( "authScript",       m_authScript       );
	config->writeEntry( "authVariables",    list               );
	config->writeEntry( "autoAuthenticate", m_autoAuthenticate );
	config->writeEntry( "sameClient",       m_sameClient       );
	config->writeEntry( "sameClientName",   m_sameClientName   );
	config->sync( );
}

//	slotErrorTimeout
//	================

void Authenticate::slotErrorTimeout( )
{	KMessageBox::sorry( m_client->listView, m_errorText, m_errorCaption );
}

//	slotProcessExited
//	=================

void Authenticate::slotProcessExited( KProcess* process )
{	kdDebug( ) << "authentication process - normalExit = "
			   << process->normalExit( )
			   << ", exitStatus = "
			   << process->exitStatus( )
			   << endl;

	emit( enableAuthButton( ));
	emit( authenticationFinished( m_client, process->normalExit( ) && !process->exitStatus( )));

	delete process;
	m_authScriptClient->locked = false;
}

//	slotReceivedStderr
//	==================

void Authenticate::slotReceivedStderr( KProcess* /* process */, char* buffer, int length )
{	createBuffer( length + 1 );
	strncpy( m_buffer, buffer, length );
	m_buffer[ length ] = '\0';
// 	kdDebug( ) << "auth stderr >" << m_buffer << "<" << endl;
}

//	slotReceivedStdout
//	==================

void Authenticate::slotReceivedStdout( KProcess* /* process */, char* buffer, int length )
{	createBuffer( length + 1 );
	strncpy( m_buffer, buffer, length );
	m_buffer[ length ] = '\0';
// 	kdDebug( ) << "auth stdout >" << m_buffer << "<" << endl;
}

//	slotWalletChanged
//	=================

void Authenticate::slotWalletChanged( Wallet* wallet )
{	m_wallet = wallet;
	saveSettings( );
}

//	slotWroteStdin
//	==============

void Authenticate::slotWroteStdin( KProcess* process )
{	switch( processScriptLine( ))
	{	case -1:
			process->writeStdin( m_authText, m_authText.length( ));
// 			kdDebug( ) << "auth stdin >" << m_authTextMasked.ascii( ) << "<" << endl;
			break;

		case 0:
			process->closeStdin( );
// 			kdDebug( ) << "auth stdin eof" << endl;
			break;

		case 1:
// 			kdDebug( ) << "auth stdin nil" << endl;
			break;

		default:
			Q_ASSERT( false );
	}
}
