/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _AUTHENTICATE_H_
#define _AUTHENTICATE_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qobject.h>
#include <qptrlist.h>
#include <qstring.h>
#include <qstringlist.h>

#include <kwallet.h>

#include "global.h"
#include "authvariable.h"

using namespace KWallet;

//	forward class definitions
//	=========================

class  KProcess;
struct passwd;
class  QWidget;

//	Authenticate class definition
//	=============================

class Authenticate : public QObject
{	friend class AuthConfigDlg;

	Q_OBJECT

	public:
		Authenticate( const CPtr client, Wallet* wallet, QObject* parent, const char* name );
	   ~Authenticate( );

		bool authenticate( );
		bool authReqd( ) const { return m_authReqd; }
		bool autoAuthenticate( ) const { return m_autoAuthenticate; }
		void configure( );
		void saveSettings( );

	signals:
		void authenticationFinished( const CPtr, const bool );
		void authenticationStarted( const CPtr );
		void disableAuthButton( );
		void enableAuthButton( );

	public slots:
		void slotWalletChanged( Wallet* wallet );

	private slots:
		void slotErrorTimeout( );
		void slotProcessExited( KProcess* process );
		void slotReceivedStdout( KProcess* process, char* buffer, int length );
		void slotReceivedStderr( KProcess* process, char* buffer, int length );
		void slotWroteStdin( KProcess* process );

	private:
		void createBuffer( const uint length );
		int  processScriptLine( );
		void readSettings( );

		bool                   m_authDataIsCrap;
		bool                   m_authReqd;
		QStringList            m_authScript;
		CPtr                   m_authScriptClient;
		ushort                 m_authScriptIndex;
		QString                m_authText;
		QString                m_authTextMasked;
		QPtrList<AuthVariable> m_authVariables;
		bool                   m_autoAuthenticate;
		string                 m_buffer;
		uint                   m_bufferLength;
		CPtr                   m_client;
		QString                m_errorCaption;
		QString                m_errorText;
		struct passwd*         m_passwdStuff;
		bool                   m_sameClient;
		QString                m_sameClientName;
		Wallet*                m_wallet;
};

#endif	// _AUTHENTICATE_H_
