/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qstringlist.h>
#include "authvariable.h"

//	constructor
//	===========

AuthVariable::AuthVariable( const AuthVariable* authVar )
: m_name( authVar->m_name ),
  m_secure( authVar->m_secure ),
  m_value( authVar->m_value ),
  m_valueMasked( authVar->m_valueMasked )
{
}

//	constructor
//	===========

AuthVariable::AuthVariable( const QString& text )
{	QStringList list = QStringList::split( "|", text );
	m_name   = list[ 0 ];
	m_secure = list[ 1 ] == "true";
	m_value  = list[ 2 ];

	buildValueMasked( );
}

//	constructor
//	===========

AuthVariable::AuthVariable( const QString& name, const bool secure, const QString& value )
: m_name( name ),
  m_secure( secure ),
  m_value( value )
{	buildValueMasked( );
}

//	buildValueMasked
//	================

void AuthVariable::buildValueMasked( )
{	if( !m_secure )
	{	m_valueMasked = m_value;
		return;
	}

	m_valueMasked = QString::null;

	for( ushort i = 0; i < m_value.length( ); i++ )
		m_valueMasked += '*';
}

//	toString
//	========

QString AuthVariable::toString( ) const
{	return m_name + "|" + (m_secure ? "true" : "false") + "|" + (m_secure ? m_valueMasked : m_value);
}

//	value
//	=====

void AuthVariable::value( const QString text )
{	m_value = text;
	buildValueMasked( );
}
