/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <kcombobox.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klocale.h>

#include "clientnamedlg.h"
#include "gui_rpc_client.h"

//	constructor
//	===========

ClientNameDlg::ClientNameDlg( const QPtrList<Client>& clients,
							  const Action   action,
							  const QString& caption,
							  QWidget*       parent,
							  const char*    name )
: KDialogBase( Plain, caption, Ok | Cancel, Ok, parent, name )
{	createLayout( clients, action );
	resize( 600, 180 );
}

//	clientName
//	==========

QString ClientNameDlg::clientName( ) const
{	return m_nameCombo->currentText( );
}

//	createLayout
//	============

void ClientNameDlg::createLayout( const QPtrList<Client>& clients, const Action action )
{	QIconSet     clearIconSet      = KGlobal::iconLoader( )->loadIconSet( "locationbar_erase", KIcon::Small );
	QPushButton* clearAuthButton   = new QPushButton( clearIconSet, NULL, plainPage( ), "clear auth button" );
	QPushButton* clearNameButton   = new QPushButton( clearIconSet, NULL, plainPage( ), "clear name button" );
	QPushButton* clearPortNoButton = new QPushButton( clearIconSet, NULL, plainPage( ), "clear port no button" );
	QGridLayout* layout            = new QGridLayout( plainPage( ));
	m_authCombo                    = new KComboBox( true, plainPage( ), "rpx auth combo box" );
	m_nameCombo                    = new KComboBox( true, plainPage( ), "name combo box" );
	m_portNoCombo                  = new KComboBox( true, plainPage( ), "port no combo box" );

	byte row = 0;

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Client name: " ), plainPage( ), "name label" ), row, 1, Qt::AlignRight );
	layout->addWidget( clearNameButton, row, 3 );
	layout->addWidget( m_nameCombo, row++, 4 );
	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Port number: " ), plainPage( ), "port no label" ), row, 1, Qt::AlignRight );
	layout->addWidget( clearPortNoButton, row, 3 );
	layout->addWidget( m_portNoCombo, row++, 4 );
	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "GUI RPC Authorisation: " ), plainPage( ), "auth label" ), row, 1, Qt::AlignRight );
	layout->addWidget( clearAuthButton, row, 3 );
	layout->addWidget( m_authCombo, row++, 4 );
	layout->setRowStretch( row++, 10 );

	layout->setColStretch( 4, 1 );

	setTabOrder( clearNameButton,   m_nameCombo       );
	setTabOrder( m_nameCombo,       clearPortNoButton );
	setTabOrder( clearPortNoButton, m_portNoCombo     );
	setTabOrder( m_portNoCombo,     clearAuthButton   );
	setTabOrder( clearAuthButton,   m_authCombo       );
	m_nameCombo->setFocus( );

	QPtrListIterator<Client> it( clients );
	CPtr client;

	for( ; (client = it.current( )) != NULL; ++it )
		switch( action )
		{	case Connect:
				if( !client->connected )
				{	m_authCombo->insertItem( client->rpcAuth );
					m_nameCombo->insertItem( client->clientName );
					m_portNoCombo->insertItem( QString::number( client->portNo ));
				}

				break;

			case Delete:
				m_authCombo->insertItem( client->rpcAuth );
				m_nameCombo->insertItem( client->clientName );
				m_portNoCombo->insertItem( QString::number( client->portNo ));
				break;

			case Disconnect:
				if( client->connected )
				{	m_authCombo->insertItem( client->rpcAuth );
					m_nameCombo->insertItem( client->clientName );
					m_portNoCombo->insertItem( QString::number( client->portNo ));
				}

				break;
		}

	connect( clearAuthButton,   SIGNAL( clicked( )),                    SLOT( slotClearAuthClicked( )));
	connect( clearNameButton,   SIGNAL( clicked( )),                    SLOT( slotClearNameClicked( )));
	connect( clearPortNoButton, SIGNAL( clicked( )),                    SLOT( slotClearPortNoClicked( )));
	connect( m_authCombo,       SIGNAL( activated( int )),              SLOT( slotAuthActivated( int )));
	connect( m_authCombo,       SIGNAL( textChanged( const QString& )), SLOT( slotAuthTextChanged( const QString& )));
	connect( m_nameCombo,       SIGNAL( activated( int )),              SLOT( slotNameActivated( int )));
	connect( m_nameCombo,       SIGNAL( textChanged( const QString& )), SLOT( slotNameTextChanged( const QString& )));
	connect( m_portNoCombo,     SIGNAL( activated( int )),              SLOT( slotPortNoActivated( int )));
	connect( m_portNoCombo,     SIGNAL( textChanged( const QString& )), SLOT( slotPortNoTextChanged( const QString& )));

	if( action != Connect )
	{	m_authCombo->setEnabled( false );
		m_portNoCombo->setEnabled( false );
	}
}

//	portNo
//	======

ushort ClientNameDlg::portNo( ) const
{	return m_portNoCombo->currentText( ).toUShort( );
}

//	rpcAuth
//	=======

QString ClientNameDlg::rpcAuth( ) const
{	return m_authCombo->currentText( );
}

//	slotAuthActivated
//	=================

void ClientNameDlg::slotAuthActivated( int index )
{	if( m_authCombo->count( ))
	{	m_nameCombo->setCurrentItem( index );
		m_portNoCombo->setCurrentItem( index );
	}
}

//	slotAuthTextChanged
//	===================

void ClientNameDlg::slotAuthTextChanged( const QString& text )
{	if( m_authCombo->count( ))
	{	m_authCombo->changeItem( text, m_authCombo->currentItem( ));
	}
}

//	slotClearAuthClicked
//	====================

void ClientNameDlg::slotClearAuthClicked( )
{	m_authCombo->setCurrentText( "" );
}

//	slotClearNameClicked
//	====================

void ClientNameDlg::slotClearNameClicked( )
{	m_nameCombo->setCurrentText( "" );
}

//	slotClearPortNoClicked
//	======================

void ClientNameDlg::slotClearPortNoClicked( )
{	m_portNoCombo->setCurrentText( "" );
}

//	slotNameActivated
//	=================

void ClientNameDlg::slotNameActivated( int index )
{	if( m_authCombo->count( ))
	{	m_authCombo->setCurrentItem( index );
		m_portNoCombo->setCurrentItem( index );
	}
}

//	slotNameTextChanged
//	===================

void ClientNameDlg::slotNameTextChanged( const QString& text )
{	if( m_nameCombo->count( ))
		m_nameCombo->changeItem( text, m_nameCombo->currentItem( ));
}

//	slotPortNoActivated
//	===================

void ClientNameDlg::slotPortNoActivated( int index )
{	m_authCombo->setCurrentItem( index );
	m_nameCombo->setCurrentItem( index );
}

//	slotPortNoTextChanged
//	=====================

void ClientNameDlg::slotPortNoTextChanged( const QString& text )
{	if( m_portNoCombo->count( ))
		m_portNoCombo->changeItem( text, m_portNoCombo->currentItem( ));
}
