/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>

#include <kapplication.h>
#include <kconfig.h>
#include <klistview.h>
#include <klocale.h>

#include "disconnecteddlg.h"

//	constructor
//	===========

DisconnectedDlg::DisconnectedDlg( QWidget* parent, const char* name )
: KDialogBase( Plain, i18n( "Disconnected clients" ), User1 | User2 | Cancel, User2, parent, name )
{	enableButton( User1, false );
	setButtonText( User1, "Retry" );
	setButtonText( User2, "Retry all" );
	setModal( false );

	createLayout( );
}

//	addClient
//	=========

DisconnectedDlg::CDPtr DisconnectedDlg::addClient( const CPtr client )
{	for( CDPtr clientDets = m_clients.first( ); clientDets != NULL; clientDets = m_clients.next( ))
		if( clientDets->client == client )
			return clientDets;

	CDPtr clientDets   = new ClientDets;
	clientDets->client = client;
	clientDets->item   = new KListViewItem( m_clientView, client->clientName, "", "", client->reason );

	m_clients.append( clientDets );

	return clientDets;
}

//	createLayout
//	============

void DisconnectedDlg::createLayout( )
{	m_clientView = new KListView( plainPage( ), "disconnected client view" );
	m_layout     = new QGridLayout( plainPage( ));
	byte row     = 0;

	m_layout->setRowStretch( row++,  1 );
	m_layout->addWidget( m_clientView, row, 1 );
	m_layout->setRowStretch( row++, 10 );
	m_layout->setRowStretch( row++,  1 );

	m_layout->setColStretch( 0,  1 );
	m_layout->setColStretch( 1, 10 );
	m_layout->setColStretch( 2,  1 );

	m_clientView->addColumn( "Client"  );
	m_clientView->addColumn( "Retries" );
	m_clientView->addColumn( "Status"  );
	m_clientView->addColumn( "Reason"  );

	m_clientView->setAllColumnsShowFocus( true );
	m_clientView->setColumnAlignment( 1, Qt::AlignRight );
	m_clientView->setSelectionMode( QListView::Extended );
	m_clientView->setShowSortIndicator( true );

	m_clientView->restoreLayout( kapp->config( ), "DisconnectedClientsView" );
	connect( m_clientView, SIGNAL( selectionChanged( )), SLOT( slotSelectionChanged( )));
	resize( 600, 300 );
}

//	findOrCreateClient
//	==================

DisconnectedDlg::CDPtr DisconnectedDlg::findOrCreateClient( const CPtr client )
{	CDPtr clientDets;

	for( clientDets = m_clients.first( ); clientDets != NULL; clientDets = m_clients.next( ))
		if( clientDets->client == client )
			return clientDets;

	clientDets = addClient( client );
	return clientDets;
}

//	removeClient
//	============

void DisconnectedDlg::removeClient( const CPtr client )
{	CDPtr clientDets = findOrCreateClient( client );

	if( clientDets == NULL )
		return;

	delete clientDets->item;
	m_clients.removeRef( clientDets );
	delete clientDets;

	if( !m_clients.count( ))
		slotCancel( );
}

//	slotCancel
//	==========

void DisconnectedDlg::slotCancel( )
{	m_clientView->saveLayout( kapp->config( ), "DisconnectedClientsView" );
	KDialogBase::slotCancel( );
}

//	slotSelectionChanged
//	====================

void DisconnectedDlg::slotSelectionChanged( )
{	for( QListViewItem* item= m_clientView->firstChild( ); item != NULL; item = item->nextSibling( ))
		if( item->isSelected( ))
		{	enableButton( User1, true );
			return;
		}

	enableButton( User1, false );
}

//	slotUser1
//	=========

void DisconnectedDlg::slotUser1( )
{	for( CDPtr clientDets = m_clients.first( ); clientDets != NULL; clientDets = m_clients.next( ))
		if( clientDets->item->isSelected( ))
			emit( retryConnection( clientDets->client ));
}

//	slotUser2
//	=========

void DisconnectedDlg::slotUser2( )
{	for( CDPtr clientDets = m_clients.first( ); clientDets != NULL; clientDets = m_clients.next( ))
		emit( retryConnection( clientDets->client ));
}

//	updateStatus
//	============

void DisconnectedDlg::updateStatus( const CPtr client, const QString& text )
{	CDPtr clientDets = findOrCreateClient( client );
	clientDets->item->setText( 1, QString::number( client->retryCount ));
	clientDets->item->setText( 2, text );
}

//	updateRetryCount
//	================

void DisconnectedDlg::updateRetryCount( const CPtr client )
{	CDPtr clientDets = findOrCreateClient( client );
	clientDets->item->setText( 1, QString::number( client->retryCount ));
}
