/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _GLOBAL_H_
#define _GLOBAL_H_

#include <qcolor.h>
#include <qstring.h>
#include "gui_rpc_client.h"

//	forward class definitions
//	=========================

class Authenticate;
class ClientButton;
class MainWidgetView;
class MessageFilter;
class MessagesFilterCheckBox;
class MessagesViewItem;
class ProjectsViewItem;
class StatisticsControlData;
class StatisticsData;
class StatisticsProjectData;
class TransfersControlView;

//	typedef's
//	=========

typedef bool*          boolPtr;
typedef unsigned char  byte;
typedef byte*          bytePtr;
typedef double*        dblPtr;
typedef dblPtr*        dblPPtr;
typedef char*          string;
typedef unsigned int   uint;
typedef uint*          uiPtr;
typedef unsigned short ushort;
typedef ushort*        usPtr;
typedef usPtr*         usPPtr;

typedef Authenticate*           APtr;
typedef ClientButton*           CBPtr;
typedef MainWidgetView*         MWVPtr;
typedef MessageFilter*          MFPtr;
typedef MessagesFilterCheckBox* MFCBPtr;
typedef MessagesViewItem*       MVIPtr;
typedef ProjectsViewItem*       PVIPtr;
typedef StatisticsControlData*  SCDPtr;
typedef SCDPtr*                 SCDPPtr;
typedef StatisticsData*         SDPtr;
typedef StatisticsProjectData*  SPDPtr;

//	client details definition
//	=========================

typedef struct
{	APtr       authenticate;
	CBPtr      button;
	QString    clientName;
	bool       connected;
	MWVPtr     listView;
	bool       locked;
	bool       manual;
	ushort     portNo;
	QString    projectName;
	QString    reason;
	int        retryCount;
	time_t     retryTime;
	QString    rpcAuth;
	RPC_CLIENT rpcClient;
	CC_STATE   state;
} Client, *CPtr;

//	graph types
//	===========

enum GraphType
{	HostAvg,
	HostTotal,
	UserAvg,
	UserTotal
};

//	message filter actions
//	======================

enum FilterAction
{	FilterActionColour,
	FilterActionHide,
	FilterActionShow
};

//	standard filter stuff
//	=====================

typedef struct
{	FilterAction alertErrorAction;
	QColor       alertErrorColour;
	FilterAction alertInfoAction;
	QColor       alertInfoColour;
	FilterAction errorAction;
	QColor       errorColour;
	FilterAction infoAction;
	QColor       infoColour;
	FilterAction userAction;
	QColor       userColour;
} StdFilter, *SFPtr;

//	macros
//	======

#define _equal_( a, b )	(fabs( (a) - (b) ) < 0.1E-6)
#define _max_( a, b )	((a) > (b) ? (a) : (b))
#define _min_( a, b )	((a) < (b) ? (a) : (b))
#define _sign_( a )		((a) < 0 ? -1 : 1)

#define GetFilters		MainWidget::self( )->getFilters( )

//	constant sh1t
//	=============

const char StandardFilterName[] = "Standard";


#endif	// _GLOBAL_H_
