/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qvbox.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kprocess.h>

#include "guiurlbutton.h"
#include "guiurlview.h"

//	constructor
//	===========

GuiURLView::GuiURLView( const PROJECT* project, QWidget* parent, const char* name )
: QScrollView( parent, name ),
  m_projectName( project->project_name.c_str( ))
{	byte spacing = fontMetrics( ).width( "XX" );

	QVBox* vBox = new QVBox( viewport( ));
	vBox->setEraseColor( viewport( )->eraseColor( ));
	vBox->setMargin( spacing / 2 );
	vBox->setSpacing( spacing );

	addChild( vBox );

	for( uint i = 0; i < project->gui_urls.size( ); i++ )
	{	GuiURLButton* button = new GuiURLButton( project->gui_urls[ i ].name.c_str( ),
												 project->gui_urls[ i ].description.c_str( ),
												 project->gui_urls[ i ].url.c_str( ),
												 vBox,
												 "gui url button" );
		connect( button, SIGNAL( clicked( const QString& )), SLOT( slotGuiURLClicked( const QString& )));
	}
}

//	slotGuiURLClicked
//	=================

void GuiURLView::slotGuiURLClicked( const QString& url )
{	string path = getenv( "PATH" );

	if( path == NULL )
	{	KMessageBox::sorry( this, i18n( "You do not seem to have a \"PATH\" environment variable.\nThis is VERY strange!\nAs a result, none of these buttons are of any use to you." ), i18n( "No $PATH" ));
		return;
	}

	QStringList paths = QStringList::split( ':', path );
	QString kfmclient;

	for( QStringList::Iterator it = paths.begin( ); it != paths.end( ); ++it )
	{	kfmclient = *it + "/kfmclient";

		if( QFile::exists( kfmclient ))
			break;
	}

	if( kfmclient.isEmpty( ))
	{	KMessageBox::sorry( this, i18n( "You do not seem to have \"kfmclient\" installed.\nThis is VERY strange!\nAs a result, none of these buttons are of any use to you." ), i18n( "No $PATH" ));
		return;
	}

	KProcess* proccess = new KProcess;
	*proccess << kfmclient;
	*proccess << "newTab";
	*proccess << url;
	proccess->start( KProcess::DontCare );
}
