/***************************************************************************
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qtimer.h>

#include <kaction.h>
#include <kactionclasses.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <ksystemtray.h>

#include "kboincmgr.h"
#include "mainwidget.h"
#include "tooltip.h"

//	constructor
//	===========

KBoincMgr::KBoincMgr( )
: KMainWindow( 0, "KBoincMgr" ),
  m_dockWidget( new KSystemTray( this, "system tray" )),
  m_mainWidget( new MainWidget( this, "view" ))
{	setCentralWidget( m_mainWidget );
	readSettings(  );
	createMenu( );
	createStatusBarStuff( );
	setAutoSaveSettings( );

	m_dockWidget->setPixmap( m_dockWidget->loadIcon( "kboincmgr" ));
	m_dockWidget->show( );

	new ToolTip( m_mainWidget, m_dockWidget );

	KAction* quit = m_dockWidget->actionCollection( )->action( "file_quit" );
	quit->disconnect( );
	connect( quit, SIGNAL( activated( )), kapp, SLOT( quit( )));

	connect( m_mainWidget,
			 SIGNAL( changeCurrentClient( const QString& )),
			 SLOT( slotChangeCurrentClient( const QString& )));

	connect( m_mainWidget,
			 SIGNAL( hasKonsole( const bool )),
			 SLOT( slotHasKonsole( const bool )));

	connect( m_mainWidget,
			 SIGNAL( statusBarMessage( const QString& )),
			 SLOT( slotStatusBarMessage( const QString& )));

	QTimer::singleShot( 0, this, SLOT( slotFinaliseInitialisation( )));
}

//	closeEvent
//	==========

void KBoincMgr::closeEvent( QCloseEvent* event )
{	hide( );
	event->ignore( );
}

//	connectToClient
//	===============

void KBoincMgr::connectToClient( const QString& clientName, const ushort portNo, const QString rpcAuth )
{	m_mainWidget->connectToClient( clientName, portNo, rpcAuth );
}

//	createMenu
//	==========

void KBoincMgr::createMenu( )
{	KStdAction::configureToolbars( this, SLOT( slotConfigureToolbars( )), actionCollection( ));
	KStdAction::keyBindings( this, SLOT( slotConfigureShortcuts( )), actionCollection( ));
	KStdAction::preferences( m_mainWidget, SLOT( slotPreferences( )), actionCollection( ));
	KStdAction::quit( kapp, SLOT( quit( )), actionCollection( ));

	m_konsoleClose = new KAction( i18n( "Close Konsole" ),
								  "konsole_close",
								  0,
								  m_mainWidget,
								  SLOT( slotKonsoleClose( )),
								  actionCollection( ),
								  "window_konsole_close" );

	m_konsoleOpen  = new KAction( i18n( "Open Konsole" ),
								  "konsole_open",
								  0,
								  m_mainWidget,
								  SLOT( slotKonsoleOpen( )),
								  actionCollection( ),
								  "window_konsole_open" );

	new KAction( i18n( "Connect client..." ),
					   "filenew",
						CTRL + Key_N,
						m_mainWidget,
						SLOT( slotConnectClient( )),
						actionCollection( ),
					   "file_connect_client" );

	new KAction( i18n( "Delete client..." ),
					   "button_fewer",
						0,
						m_mainWidget,
						SLOT( slotDeleteClient( )),
						actionCollection( ),
						"file_delete_client" );

	new KAction( i18n( "Disconnect client..." ),
					   "fileclose",
						CTRL + Key_W,
						m_mainWidget,
						SLOT( slotDisconnectClient( )),
						actionCollection( ),
					   "file_disconnect_client" );

	new KAction( i18n( "Update now" ),
					   "redo",
						Key_F5,
						m_mainWidget,
						SLOT( slotUpdateNow( )),
						actionCollection( ),
					   "window_update_now" );

	m_konsoleClose->setEnabled( false );
	m_konsoleOpen->setEnabled( false );

	createStandardStatusBarAction( );
	setStandardToolBarMenuEnabled( true );
	toolBar( )->show( );

#ifdef _DEBUG
	createGUI( QDir::homeDirPath( ) + "/.kde/share/apps/kboincmgr/kboincmgrui.rc" );
#else
	createGUI( );
#endif
}

//	createStatusBarStuff
//	====================

void KBoincMgr::createStatusBarStuff( )
{	QPushButton* updateButton = new QPushButton( i18n( "Update now" ), statusBar( ));
	statusBar( )->addWidget( updateButton, 0, true );
	connect( updateButton, SIGNAL( clicked( )), m_mainWidget, SLOT( slotUpdateNow( )));
}

//	queryExit
//	=========

bool KBoincMgr::queryExit( )
{	saveSettings( );
	m_mainWidget->wrapUp( );
	return true;
}

//	readSettings
//	============

void KBoincMgr::readSettings( )
{	m_mainWidget->readSettings( );

	KConfig* config = kapp->config( );
	config->setGroup( "General" );
	m_isVisible = config->readBoolEntry( "visible", false );
}

//	saveSettings
//	============

void KBoincMgr::saveSettings( )
{	m_mainWidget->saveSettings( );

	KConfig* config = kapp->config( );
	config->setGroup( "General" );
	config->writeEntry( "visible", isVisible( ));
}

//	slotChangeCurrentClient
//	=======================

void KBoincMgr::slotChangeCurrentClient( const QString& clientName )
{	setCaption( clientName );
}

//	slotConfigureShortcuts
//	======================

void KBoincMgr::slotConfigureShortcuts( )
{	KKeyDialog::configure( actionCollection( ), this);
}

//	slotConfigureToolbars
//	=====================

void KBoincMgr::slotConfigureToolbars( )
{	saveMainWindowSettings( KGlobal::config( ), autoSaveGroup( ));

	KEditToolbar dlg( actionCollection( ));
	connect( &dlg, SIGNAL( newToolbarConfig( )),this, SLOT( slotNewToolbarConfig( )));
	dlg.exec( );
}

//	slotFinaliseInitialisation
//	==========================

void KBoincMgr::slotFinaliseInitialisation( )
{	if( m_isVisible )
		show( );
	else
		hide( );
}

//	slotHasKonsole
//	==============

void KBoincMgr::slotHasKonsole( const bool hasConsole )
{	m_konsoleClose->setEnabled( hasConsole );
	m_konsoleOpen->setEnabled( !hasConsole );
}

//	slotNewToolbarConfig
//	====================

void KBoincMgr::slotNewToolbarConfig( )
{
#ifdef _DEBUG
	createGUI( QDir::homeDirPath( ) + "/.kde/share/apps/kboincmgr/kboincmgrui.rc" );
#else
	createGUI( );
#endif

	applyMainWindowSettings( KGlobal::config( ), autoSaveGroup( ));
}

//	slotStatusBarMessage
//	====================

void KBoincMgr::slotStatusBarMessage( const QString& text )
{	statusBar( )->message( text );
}

#include "kboincmgr.moc"
