/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qvbox.h>

#include "clientbutton.h"
#include "mainwidgetcontrol.h"

//	constructor
//	===========

MainWidgetControl::MainWidgetControl( QWidget* parent, const char* name )
: QScrollView( parent, name ),
  m_currentClient( NULL )
{	byte spacing = fontMetrics( ).width( "XX" );

	m_vBox = new QVBox( viewport( ));
	m_vBox->setEraseColor( viewport( )->eraseColor( ));
	m_vBox->setMargin( spacing / 2 );
	m_vBox->setSpacing( spacing );

	addChild( m_vBox );
}

//	addClient
//	=========

void MainWidgetControl::addClient( CPtr client )
{	client->button = new ClientButton( client, m_vBox, "client button" );
	client->button->show( );

	connect( client->button, SIGNAL( clicked( const CPtr )), SLOT( slotClientClicked( const CPtr )));
}

//	removeClient
//	============

void MainWidgetControl::removeClient( CPtr client )
{	delete client->button;
	client->button = NULL;

	if( m_currentClient == client )
		m_currentClient = NULL;
}

//	setCurrentClient
//	================

void MainWidgetControl::setCurrentClient( CPtr currentClient )
{	if( m_currentClient != NULL )
		m_currentClient->button->setCurrentClient( false );

	m_currentClient = currentClient;

	if( !m_currentClient->connected )
		return;

	if( m_currentClient != NULL )
		m_currentClient->button->setCurrentClient( true );
}

//	slotClientClicked
//	=================

void MainWidgetControl::slotClientClicked( const CPtr client )
{	if( m_currentClient != NULL )
		m_currentClient->button->setCurrentClient( false );

	m_currentClient = client;
	m_currentClient->button->setCurrentClient( true );
	emit( changeCurrentClient( m_currentClient ));
}
