/***************************************************************************
 *   Copyright (C) 2006 by Kevin Gilbert   *
 *   kev.gilbert@cdu.edu.au   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _MAINWIDGETVIEW_H_
#define _MAINWIDGETVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <ktabwidget.h>
#include <kwallet.h>
#include "global.h"

using namespace KWallet;

//	forward class definitions
//	=========================

class CC_STATE;
class QFont;
class RPC_CLIENT;
class MessagesView;
class ProjectsView;
class QResizeEvent;
class QShowEvent;
class QStrList;
class ResultsView;
class StatisticsView;
class TerminalInterface;
class TransfersView;

//	MainWidgetView class definition
//	===============================

class MainWidgetView : public KTabWidget
{	Q_OBJECT

	public:
		MainWidgetView( const CPtr   client,
						const QFont& font,
						QWidget*     parent,
						const char*  name );

		static bool    getState( const CPtr client );
		static QString secondsToString( double seconds );

		void    changeFont( const QFont& font );
		void    filtersUpdated( );
		void    konsoleClose( );
		void    konsoleOpen( );
		void    saveSettings( const bool clientSpecific );
		QString toolTipText( );
		bool    update( );
		void    updateProjects( );

	signals:
		void hasKonsole( const bool );

	private slots:
		void slotKonsolePartDestroyed( );

	private:
		void createKonsolePart( );
		void polish( );
		void showEvent( QShowEvent* event );

		CPtr               m_client;
		bool               m_hasKonsole;
		bool               m_haveStateData;
		QWidget*           m_konsoleWidget;
		MessagesView*      m_messagesView;
		ProjectsView*      m_projectsView;
		ResultsView*       m_resultsView;
		StatisticsView*    m_statisticsView;
		TerminalInterface* m_termInterface;
		TransfersView*     m_transfersView;
};

#endif // _MAINWIDGETVIEW_H_
