/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006-2007 by Kevin Gilbert                              *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qstringlist.h>
#include "messagefilter.h"

//	constructor
//	===========

MessageFilter::MessageFilter( )
: m_action( FilterActionColour ),
  m_caseSensitive( false ),
  m_colour( QColor( 0, 0, 0 ) ),
  m_filterName( StandardFilterName ),
  m_global( true ),
  m_isRegexp( false ),
  m_isStandardFilter( true )
{
}

//	constructor
//	===========

MessageFilter::MessageFilter( const MFPtr filter )
: m_action( filter->m_action ),
  m_caseSensitive( filter->m_caseSensitive ),
  m_clientName( filter->m_clientName ),
  m_colour( filter->m_colour ),
  m_filterName( filter->m_filterName ),
  m_filterText( filter->m_filterText ),
  m_global( filter->m_global ),
  m_isRegexp( filter->m_isRegexp ),
  m_isStandardFilter( filter->m_isStandardFilter )
{
}

//	constructor
//	===========

MessageFilter::MessageFilter( const QString& text )
: m_isStandardFilter( false )
{	QStringList list   = QStringList::split( "\034", text );

	if( list.count( ) != 8 )
	{	m_action        = FilterActionColour;
		m_caseSensitive = false;
		m_clientName    = "**** unknown ****";
		m_colour        = QColor( 255, 0, 0 );
		m_filterName    = "new filter";
		m_global        = false;
		m_isRegexp      = false;
	}
	else
	{	m_action        = FilterAction( list[ 0 ].toUInt( ));
		m_caseSensitive = stringToBool( list[ 1 ]);
		m_clientName    = list[ 2 ];
		m_colour        = list[ 3 ];
		m_filterName    = list[ 4 ];
		m_filterText    = list[ 5 ];
		m_global        = stringToBool( list[ 6 ]);
		m_isRegexp      = stringToBool( list[ 7 ]);
	}
}

//	boolToString
//	============

QString MessageFilter::boolToString( const bool b ) const
{	return b ? "true" : "false";
}

//	stringToBool
//	============

bool MessageFilter::stringToBool( const QString& text ) const
{	return text == "true";
}

//	toString
//	========

QString MessageFilter::toString( ) const
{	QString text = QString::number( m_action )     + "\034" +
				   boolToString( m_caseSensitive ) + "\034" +
				   m_clientName                    + "\034" +
				   m_colour.name( )                + "\034" +
				   m_filterName                    + "\034" +
				   m_filterText                    + "\034" +
				   boolToString( m_global )        + "\034" +
				   boolToString( m_isRegexp );
	return text;
}
