/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006-2007 by Kevin Gilbert                              *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qradiobutton.h>

#include <kcolordialog.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "mainwidget.h"
#include "messagefilter.h"
#include "messagefilterdlg.h"

//	constructor
//	===========

MessageFilterDlg::MessageFilterDlg( const MFPtr filter, const EditType editType, QWidget* parent, const char* name )
: KDialogBase( Plain, i18n( "Message filter configuration" ), Ok | Apply | Cancel, Ok, parent, name ),
  m_editType( editType ),
  m_filter( filter )
{	createLayout( );
}

//	destructor
//	=========

MessageFilterDlg::~MessageFilterDlg( )
{	delete m_colourPixmap;
}

//	apply
//	=====

bool MessageFilterDlg::apply( )
{	QString filterName = m_filterNameEdit->text( );
	QString filterText = m_filterTextEdit->text( );

	if( filterName.isEmpty( ))
	{	KMessageBox::sorry( NULL, i18n( "Filter name cannot be empty" ), i18n( "Filter name empty" ));
		return false;
	}

	if( filterName.contains( '&' ))
	{	KMessageBox::sorry( NULL, i18n( "Filter name cannot contains the character '&'" ), i18n( "Filter name illegal" ));
		return false;
	}

	for( MFPtr filter = GetFilters.first( ); filter != NULL; filter = GetFilters.next( ))
		if( filter != m_filter && filterName == filter->filterName( ))
		{	KMessageBox::sorry( NULL, i18n( "Duplicate filter name" ), i18n( "Duplicate filter name" ));
			return false;
		}

	if( filterText.isEmpty( ))
	{	KMessageBox::sorry( NULL, i18n( "Filter text cannot be empty" ), i18n( "Filter text empty" ));
		return false;
	}

	m_filter->caseSensitive( m_caseSensitiveCheckBox->isChecked( ));
	m_filter->filterName( filterName );
	m_filter->filterText( filterText );
	m_filter->isRegexp( m_regexpCheckBox->isChecked( ));
	m_filter->global( m_globalCheckBox->isChecked( ));

	if( m_actionColourButton->isChecked( ))
		m_filter->action( FilterActionColour );
	else
	if( m_actionHideButton->isChecked( ))
		m_filter->action( FilterActionHide );
	else
	if( m_actionShowButton->isChecked( ))
		m_filter->action( FilterActionShow );

	emit( applyFilter( m_filter, m_editType ));

	m_editType = TypeEdit;
	m_filterNameButton->setEnabled( false );
	m_filterNameEdit->setReadOnly( true );

	return true;
}

//	createLayout
//	============

void MessageFilterDlg::createLayout( )
{	KIconLoader* iconLoader        = KGlobal::iconLoader( );
	QIconSet     clearRightIconSet = iconLoader->loadIconSet( "locationbar_erase", KIcon::Small );

	QGridLayout* layout = new QGridLayout( plainPage( ));
	byte row = 0;

	layout->setRowStretch( row++, 10 );

	// row 1
	// =====

	layout->addWidget( new QLabel( i18n( "Filter name: " ), plainPage( ), "name label" ), row, 1, Qt::AlignRight );
	m_filterNameButton = new QPushButton( clearRightIconSet, NULL, plainPage( ), "name button" );
	m_filterNameButton->setEnabled( m_editType != TypeEdit );
	connect( m_filterNameButton, SIGNAL( clicked( )), SLOT( slotClearFilterName( )));
	layout->addWidget( m_filterNameButton, row, 2 );
	m_filterNameEdit = new KLineEdit( m_filter->filterName( ), plainPage( ), "name edit" );
	m_filterNameEdit->setReadOnly( m_editType == TypeEdit );
	layout->addWidget( m_filterNameEdit, row, 3 );

	m_globalCheckBox = new QCheckBox( i18n( "Global" ), plainPage( ), "global checkbox" );
	m_globalCheckBox->setChecked( m_filter->global( ));
	layout->addWidget( m_globalCheckBox, row, 5 );

	QButtonGroup* actionGroup = new QButtonGroup( 3, Qt::Vertical, i18n( "Action" ), plainPage( ), "action group" );
	m_actionColourButton      = new QRadioButton( i18n( "Colour" ), actionGroup, "action colour button" );
	m_actionHideButton        = new QRadioButton( i18n( "Hide"   ), actionGroup, "hide colour button"   );
	m_actionShowButton        = new QRadioButton( i18n( "Show"   ), actionGroup, "shoe colour button"   );
	layout->addMultiCellWidget( actionGroup, row, row + 2, 7, 7 );

	row++;

	// row 2
	// =====

	layout->addWidget( new QLabel( i18n( "Filter text: " ), plainPage( ), "text label" ), row, 1, Qt::AlignRight );
	m_filterTextButton = new QPushButton( clearRightIconSet, NULL, plainPage( ), "text button" );
	connect( m_filterTextButton, SIGNAL( clicked( )), SLOT( slotClearFilterText( )));
	layout->addWidget( m_filterTextButton, row, 2 );
	m_filterTextEdit = new KLineEdit( m_filter->filterText( ), plainPage( ), "text edit" );
	layout->addWidget( m_filterTextEdit, row, 3 );

	m_regexpCheckBox = new QCheckBox( i18n( "Regular expression" ), plainPage( ), "regexp checkbox" );
	m_regexpCheckBox->setChecked( m_filter->isRegexp( ));
	layout->addWidget( m_regexpCheckBox, row, 5 );

	row++;

	// row 3
	// =====

	byte colourSize = fontMetrics( ).width( "XX" );
	m_colourPixmap = new QPixmap( colourSize, colourSize );
	m_colourPixmap->fill( m_filter->colour( ));
	m_colourButton = new QPushButton( *m_colourPixmap, i18n( "Text colour" ), plainPage( ), "colour button" );
	connect( m_colourButton, SIGNAL( clicked( )), SLOT( slotColourButtonClicked( )));
	layout->addMultiCellWidget( m_colourButton, row, row, 1, 3 );

	m_caseSensitiveCheckBox = new QCheckBox( i18n( "Case sensitive" ), plainPage( ), "caseSensitive checkbox" );
	m_caseSensitiveCheckBox->setChecked( m_filter->caseSensitive( ));
	layout->addWidget( m_caseSensitiveCheckBox, row, 5 );

	row++;
	layout->setRowStretch( row++, 10 );

	layout->setColStretch( 0,  2 );
	layout->setColStretch( 3, 10 );
	layout->setColStretch( 4,  2 );
	layout->setColStretch( 6,  2 );
	layout->setColStretch( 8,  2 );

	// layout finalisation
	// ===================

	setTabOrder( m_filterNameButton,      m_filterNameEdit        );
	setTabOrder( m_filterNameEdit,        m_filterTextButton      );
	setTabOrder( m_filterTextButton,      m_filterTextEdit        );
	setTabOrder( m_filterTextEdit,        m_colourButton          );
	setTabOrder( m_colourButton,          m_globalCheckBox        );
	setTabOrder( m_globalCheckBox,        m_regexpCheckBox        );
	setTabOrder( m_regexpCheckBox,        m_caseSensitiveCheckBox );
	setTabOrder( m_caseSensitiveCheckBox, m_actionColourButton    );
	setTabOrder( m_actionColourButton,    m_actionHideButton      );
	setTabOrder( m_actionHideButton,      m_actionShowButton      );

	switch( m_filter->action( ))
	{	case FilterActionColour: m_actionColourButton->setChecked( true ); break;
		case FilterActionHide:   m_actionHideButton->setChecked( true );   break;
		case FilterActionShow:   m_actionShowButton->setChecked( true );   break;
	}
}

//	slotApply
//	=========

void MessageFilterDlg::slotApply( )
{	apply( );
}

//	slotClearFilterName
//	===================

void MessageFilterDlg::slotClearFilterName( )
{	m_filterNameEdit->clear( );
}

//	slotClearFilterText
//	===================

void MessageFilterDlg::slotClearFilterText( )
{	m_filterTextEdit->clear( );
}

//	slotColourButtonClicked
//	=======================

void MessageFilterDlg::slotColourButtonClicked( )
{	QColor colour( m_filter->colour( ));

	if( KColorDialog::getColor( colour ) != KColorDialog::Accepted )
		return;

	m_filter->colour( colour );
	m_colourPixmap->fill( colour );
	m_colourButton->setIconSet( *m_colourPixmap );
}

//	slotOk
//	======

void MessageFilterDlg::slotOk( )
{	if( apply( ))
		KDialogBase::slotOk( );
}
