/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _MESSAGESCONTROLVIEW_H_
#define _MESSAGESCONTROLVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include "global.h"
#include "messagefilter.h"
#include "messagesfiltercheckbox.h"
#include "messagefilterdlg.h"

//	forward class definitions
//	=========================

class QGrid;
class QPushButton;

//	TransfersControlView class definition
//	=====================================

class MessagesControlView : public QScrollView
{	Q_OBJECT

	public:
		MessagesControlView( const CPtr client, QWidget* parent, const char* name );

		void filtersUpdated( );
		void saveSettings( );

	signals:
		void activateFilters( QPtrList<MessageFilter>& );
		void copyAllClicked( );
		void copySelectedClicked( );

	private slots:
		void slotActivateFilter( );
		void slotEnableCopySelectedButton( const bool b );
		void slotFilterApply( const MFPtr filter, const MessageFilterDlg::EditType editType );
		void slotFilterAdd( );
		void slotFilterCopy( const MFPtr filter );
		void slotFilterDelete( const MFPtr filter );
		void slotFilterDownClicked( const MFCBPtr checkBox );
		void slotFilterEdit( const MFPtr filter );
		void slotFilterUpClicked( const MFCBPtr checkBox );
		void slotFinaliseInitialisation( );

	private:
		void    activateFilters( const QString& activeFilters );
		void    addFilter( const MFPtr filter );
		void    createLayout( );
		MFPtr   findFilter( const QString& filterName );
		QString getActiveFilters( );
		QString getAllFilters( );
		void    initFilterView( QPtrList<MessageFilter>& filters );

		bool         m_addOrCopyApplied;
		CPtr         m_client;
		QPushButton* m_copyAll;
		QPushButton* m_copySelected;
		QPushButton* m_filterButtonAdd;
		QGrid*       m_filtersGrid;

		QPtrList<MessagesFilterCheckBox> m_filterCheckBoxes;
};

#endif	// _MESSAGESCONTROLVIEW_H_
