/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006-2007 by Kevin Gilbert                              *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qevent.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>

#include <kiconloader.h>
#include <klocale.h>

#include "messagefilter.h"
#include "messagesfiltercheckbox.h"

//	constructor
//	===========

MessagesFilterCheckBox::MessagesFilterCheckBox( const MFPtr filter, QGrid* parent, const char* name )
: QCheckBox( filter->filterName( ), parent, name ),
  m_filter( filter )
{	KIconLoader* iconLoader  = KGlobal::iconLoader( );
	QIconSet     downIconSet = iconLoader->loadIconSet( "downarrow", KIcon::Small );
	QIconSet     upIconSet   = iconLoader->loadIconSet( "uparrow",   KIcon::Small );

	m_up   = new QPushButton( upIconSet,   NULL, parent, "up filter button"   );
	m_down = new QPushButton( downIconSet, NULL, parent, "down filter button" );

	show( );
	m_down->show( );
	m_up->show( );

	connect( m_down, SIGNAL( clicked( )), SLOT( slotFilterDownClicked( )));
	connect( m_up,   SIGNAL( clicked( )), SLOT( slotFilterUpClicked( )));
	connect( this,   SIGNAL( clicked( )), SLOT( slotFilterClicked( )));
}

//	destructor
//	==========

MessagesFilterCheckBox::~MessagesFilterCheckBox( )
{	delete m_down;
	delete m_up;
}

//	contextMenuEvent
//	================

void MessagesFilterCheckBox::contextMenuEvent( QContextMenuEvent* event )
{	QPopupMenu* menu = new QPopupMenu( this );

	if( m_filter->isStandardFilter( ))
		menu->insertItem( i18n( "Edit "   ) + m_filter->filterName( ), this, SLOT(slotFilterEdit( )));
	else
	{	menu->insertItem( i18n( "Copy "   ) + m_filter->filterName( ), this, SLOT(slotFilterCopy( )));
		menu->insertItem( i18n( "Edit "   ) + m_filter->filterName( ), this, SLOT(slotFilterEdit( )));
		menu->insertSeparator();
		menu->insertItem( i18n( "Delete " ) + m_filter->filterName( ), this, SLOT(slotFilterDelete( )));
	}

	menu->exec( event->globalPos( ));
}

//	enableDown
//	==========

void MessagesFilterCheckBox::enableDown( const bool enable )
{	m_down->setEnabled( enable );
}

//	enableUp
//	========

void MessagesFilterCheckBox::enableUp( const bool enable )
{	m_up->setEnabled( enable );
}

//	slotFilterClicked
//	=================

void MessagesFilterCheckBox::slotFilterClicked( )
{	emit( filterClicked( ));
}

//	slotFilterCopy
//	==============

void MessagesFilterCheckBox::slotFilterCopy( )
{	emit( filterCopy( m_filter ));
}

//	slotFilterDelete
//	================

void MessagesFilterCheckBox::slotFilterDelete( )
{	emit( filterDelete( m_filter ));

}

//	slotFilterDownClicked
//	=====================

void MessagesFilterCheckBox::slotFilterDownClicked( )
{	emit( filterDownClicked( this ));
}

//	slotFilterEdit
//	==============

void MessagesFilterCheckBox::slotFilterEdit( )
{	emit( filterEdit( m_filter ));
}

//	slotFilterUpClicked
//	===================

void MessagesFilterCheckBox::slotFilterUpClicked( )
{	emit( filterUpClicked( this ));
}
