/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _MESSAGESLISTVIEW_H_
#define _MESSAGESLISTVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qptrlist.h>

#include <klistview.h>

#include "global.h"
#include "messagefilter.h"

//	forward class definitions
//	=========================

class PROJECT;
class QTimer;
class RPC_CLIENT;

//	MessagesView class definition
//	=============================

class MessagesListView : public KListView
{	Q_OBJECT

	public:
		MessagesListView( const CPtr client, QWidget* parent, const char* name );
	   ~MessagesListView( );

		void authenticationFinished( const CPtr client, const bool successful );
		void authenticationStarted( const CPtr client );
		void autoProjectUpdateStarted( );
		bool update( );

	signals:
		void doProjectsUpdate( );
		void enableCopySelectedButton( const bool );

	public slots:
		void slotActivateFilters( QPtrList<MessageFilter>& activeFilters );
		void slotCopyAllClicked( );
		void slotCopySelectedClicked( );

	private slots:
		void slotScrollToEnd( );
		void slotSelectionChanged( );

	private:
		bool applyFilter( const MFPtr filter, const MVIPtr item );
		void applyFilters( MVIPtr item );
		void applyStandardFilter( const MVIPtr item );
		void applyStandardFilter( const MVIPtr item, const FilterAction action, const QColor colour );
		void filtersClear( );

		const double SeqNoInc;

		CPtr    m_client;
		bool    m_haveHide;
		bool    m_haveShow;
		double  m_seqNo;
		QTimer* m_timer;

		QPtrList<MessageFilter> m_filters;
};

#endif	// _MESSAGESLISTVIEW_H_
