/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006-2007 by Kevin Gilbert                              *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>

#include <kcolordialog.h>
#include <kcombobox.h>
#include <klocale.h>

#include "mainwidget.h"
#include "messagestandarddlg.h"

//	constructor
//	===========

MessageStandardDlg::MessageStandardDlg( QWidget* parent, const char* name )
: KDialogBase( Plain, i18n( "Edit standard filter" ), Ok | Apply | Cancel, Ok, parent, name ),
  m_standardFilter( MainWidget::self( )->standardFilter( ))
{	createLayout( );
}

//	destructor
//	=========

MessageStandardDlg::~MessageStandardDlg( )
{	delete m_alertErrorPixmap;
	delete m_alertInfoPixmap;
	delete m_errorPixmap;
	delete m_infoPixmap;
	delete m_userPixmap;
}

//	createLayout
//	============

void MessageStandardDlg::createLayout( )
{	byte colourSize = fontMetrics( ).width( "XX" );
	QGridLayout* layout = new QGridLayout( plainPage( ));
	byte row = 0;

	layout->setRowStretch( row++, 10 );
	layout->addWidget( new QLabel( i18n( "Message type" ), plainPage( ), "type label"   ), row, 1 );
	layout->addWidget( new QLabel( i18n( "Colour"       ), plainPage( ), "colour label" ), row, 3 );
	layout->addWidget( new QLabel( i18n( "Action"       ), plainPage( ), "action label" ), row, 5 );
	row++;
	layout->setRowStretch( row++, 1 );

	layout->addWidget( new QLabel( i18n( "Alert error" ), plainPage( ), "alert error label" ), row, 1, Qt::AlignRight );
	m_alertErrorPixmap = new QPixmap( colourSize, colourSize );
	m_alertErrorPixmap->fill( m_standardFilter.alertErrorColour );
	m_alertErrorButton = new QPushButton( *m_alertErrorPixmap, NULL, plainPage( ), "alert error button" );
	connect( m_alertErrorButton, SIGNAL( clicked( )), SLOT( slotAlertErrorButtonClicked( )));
	layout->addWidget( m_alertErrorButton, row, 3 );
	m_alterErrorComboBox = new KComboBox( false, plainPage( ), "alert error combo box" );
	initComboBox( m_alterErrorComboBox, m_standardFilter.alertErrorAction );
	layout->addWidget( m_alterErrorComboBox, row++, 5 );
	layout->setRowStretch( row++, 1 );

	layout->addWidget( new QLabel( i18n( "Alert info" ), plainPage( ), "alert info label" ), row, 1, Qt::AlignRight );
	m_alertInfoPixmap = new QPixmap( colourSize, colourSize );
	m_alertInfoPixmap->fill( m_standardFilter.alertInfoColour );
	m_alertInfoButton = new QPushButton( *m_alertInfoPixmap, NULL, plainPage( ), "alert info button" );
	connect( m_alertInfoButton, SIGNAL( clicked( )), SLOT( slotAlertInfoButtonClicked( )));
	layout->addWidget( m_alertInfoButton, row, 3 );
	m_alterInfoComboBox = new KComboBox( false, plainPage( ), "alert info combo box" );
	initComboBox( m_alterInfoComboBox, m_standardFilter.alertInfoAction );
	layout->addWidget( m_alterInfoComboBox, row++, 5 );
	layout->setRowStretch( row++, 1 );

	layout->addWidget( new QLabel( i18n( "Error" ), plainPage( ), "error label" ), row, 1, Qt::AlignRight );
	m_errorPixmap = new QPixmap( colourSize, colourSize );
	m_errorPixmap->fill( m_standardFilter.errorColour );
	m_errorButton = new QPushButton( *m_errorPixmap, NULL, plainPage( ), "error button" );
	connect( m_errorButton, SIGNAL( clicked( )), SLOT( slotErrorButtonClicked( )));
	layout->addWidget( m_errorButton, row, 3 );
	m_errorComboBox = new KComboBox( false, plainPage( ), "error combo box" );
	initComboBox( m_errorComboBox, m_standardFilter.errorAction );
	layout->addWidget( m_errorComboBox, row++, 5 );
	layout->setRowStretch( row++, 1 );

	layout->addWidget( new QLabel( i18n( "Info" ), plainPage( ), "info label" ), row, 1, Qt::AlignRight );
	m_infoPixmap = new QPixmap( colourSize, colourSize );
	m_infoPixmap->fill( m_standardFilter.infoColour );
	m_infoButton = new QPushButton( *m_infoPixmap, NULL, plainPage( ), "info button" );
	connect( m_infoButton, SIGNAL( clicked( )), SLOT( slotInfoButtonClicked( )));
	layout->addWidget( m_infoButton, row, 3 );
	m_infoComboBox = new KComboBox( false, plainPage( ), "info combo box" );
	initComboBox( m_infoComboBox, m_standardFilter.infoAction );
	layout->addWidget( m_infoComboBox, row++, 5 );
	layout->setRowStretch( row++, 1 );

	layout->addWidget( new QLabel( i18n( "KBoincMgr" ), plainPage( ), "kboincmgr label" ), row, 1, Qt::AlignRight );
	m_userPixmap = new QPixmap( colourSize, colourSize );
	m_userPixmap->fill( m_standardFilter.userColour );
	m_userButton = new QPushButton( *m_userPixmap, NULL, plainPage( ), "kboincmgr button" );
	connect( m_userButton, SIGNAL( clicked( )), SLOT( slotUserButtonClicked( )));
	layout->addWidget( m_userButton, row, 3 );
	m_userComboBox = new KComboBox( false, plainPage( ), "user combo box" );
	initComboBox( m_userComboBox, m_standardFilter.userAction );
	layout->addWidget( m_userComboBox, row++, 5 );
	layout->setRowStretch( row++, 1 );

	layout->setRowStretch( row++, 10 );

	layout->setColStretch( 0, 10 );
	layout->setColStretch( 2,  5 );
	layout->setColStretch( 4,  5 );
	layout->setColStretch( 6, 10 );

	// layout finalisation
	// ===================

	setTabOrder( m_alertErrorButton,   m_alterErrorComboBox );
	setTabOrder( m_alterErrorComboBox, m_alertInfoButton    );
	setTabOrder( m_alertInfoButton,    m_alterInfoComboBox  );
	setTabOrder( m_alterInfoComboBox,  m_errorButton        );
	setTabOrder( m_errorButton,        m_errorComboBox      );
	setTabOrder( m_errorComboBox,      m_infoButton         );
	setTabOrder( m_infoButton,         m_infoComboBox       );
	setTabOrder( m_infoComboBox,       m_userButton         );
	setTabOrder( m_userButton,         m_userComboBox       );
}

//	getAction
//	=========

FilterAction MessageStandardDlg::getAction( KComboBox* comboBox )
{	if( comboBox->currentText( ) == i18n( "Colour" )) return FilterActionColour;
	if( comboBox->currentText( ) == i18n( "Hide"   )) return FilterActionHide;
	if( comboBox->currentText( ) == i18n( "Show"   )) return FilterActionShow;

	fprintf( stderr, "internal error in MessageStandardDlg::getAction - unknown action \"%s\"\n", comboBox->currentText( ).ascii( ));
	abort( );
	return FilterActionColour;
}

//	initComboBox
//	============

void MessageStandardDlg::initComboBox( KComboBox* comboBox, const FilterAction action )
{	comboBox->insertItem( i18n( "Colour" ));
	comboBox->insertItem( i18n( "Hide"   ));
	comboBox->insertItem( i18n( "Show"   ));

	switch( action )
	{	case FilterActionColour: comboBox->setCurrentItem( 0 ); break;
		case FilterActionHide:   comboBox->setCurrentItem( 1 ); break;
		case FilterActionShow:   comboBox->setCurrentItem( 2 ); break;
	}
}

//	slotAlertErrorButtonClicked
//	===========================

void MessageStandardDlg::slotAlertErrorButtonClicked( )
{	if( KColorDialog::getColor( m_standardFilter.alertErrorColour ) != KColorDialog::Accepted )
		return;

	m_alertErrorPixmap->fill( m_standardFilter.alertErrorColour );
	m_alertErrorButton->setIconSet( *m_alertErrorPixmap );
}

//	slotAlertInfoButtonClicked
//	==========================

void MessageStandardDlg::slotAlertInfoButtonClicked( )
{	if( KColorDialog::getColor( m_standardFilter.alertInfoColour ) != KColorDialog::Accepted )
		return;

	m_alertInfoPixmap->fill( m_standardFilter.alertInfoColour );
	m_alertInfoButton->setIconSet( *m_alertInfoPixmap );
}

//	slotApply
//	=========

void MessageStandardDlg::slotApply( )
{	m_standardFilter.alertErrorAction = getAction( m_alterErrorComboBox );
	m_standardFilter.alertInfoAction  = getAction( m_alterInfoComboBox  );
	m_standardFilter.errorAction      = getAction( m_errorComboBox      );
	m_standardFilter.infoAction       = getAction( m_infoComboBox       );
	m_standardFilter.userAction       = getAction( m_userComboBox       );

	MainWidget::self( )->standardFilter( m_standardFilter );
}

//	slotErrorButtonClicked
//	======================

void MessageStandardDlg::slotErrorButtonClicked( )
{	if( KColorDialog::getColor( m_standardFilter.errorColour ) != KColorDialog::Accepted )
		return;

	m_errorPixmap->fill( m_standardFilter.errorColour );
	m_errorButton->setIconSet( *m_errorPixmap );
}

//	slotInfoButtonClicked
//	======================

void MessageStandardDlg::slotInfoButtonClicked( )
{	if( KColorDialog::getColor( m_standardFilter.infoColour ) != KColorDialog::Accepted )
		return;

	m_infoPixmap->fill( m_standardFilter.infoColour );
	m_infoButton->setIconSet( *m_infoPixmap );
}

//	slotOk
//	======

void MessageStandardDlg::slotOk( )
{	slotApply( );
	KDialogBase::slotOk( );
}

//	slotUserButtonClicked
//	=====================

void MessageStandardDlg::slotUserButtonClicked( )
{	if( KColorDialog::getColor( m_standardFilter.userColour ) != KColorDialog::Accepted )
		return;

	m_userPixmap->fill( m_standardFilter.userColour );
	m_userButton->setIconSet( *m_userPixmap );
}
