/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "messagescontrolview.h"
#include "messageslistview.h"
#include "messagesview.h"

//	constructor
//	===========

MessagesView::MessagesView( const CPtr client, QWidget* parent, const char* name )
: QSplitter( parent, name ),
  m_client( client ),
  m_shown( false )
{	m_controlView = new MessagesControlView( m_client, this, "messages control view" );
	m_listView    = new MessagesListView( m_client, this, "messages list view" );

	KConfig* config = kapp->config( );
	config->setGroup( "MainWindow" );
	bool synchroniseLayouts = config->readBoolEntry( "synchroniseLayouts", false );

	if( synchroniseLayouts )
		config->setGroup( "MessagesView" );
	else
		config->setGroup( "MessagesView_" + m_client->clientName );

	m_sizes = config->readIntListEntry( "viewWidths" );

	connect( m_listView,
			 SIGNAL( doProjectsUpdate( )),
			 SIGNAL( doProjectsUpdate( )));

	connect( m_listView,
			 SIGNAL( enableCopySelectedButton( const bool )),
			 m_controlView,
			 SLOT( slotEnableCopySelectedButton( const bool )));

 	connect( m_controlView,
			 SIGNAL( activateFilters( QPtrList<MessageFilter>& )),
			 m_listView,
			 SLOT( slotActivateFilters( QPtrList<MessageFilter>& )));

	connect( m_controlView,
			 SIGNAL( copyAllClicked( )),
			 m_listView,
			 SLOT( slotCopyAllClicked( )));

	connect( m_controlView,
			 SIGNAL( copySelectedClicked( )),
			 m_listView,
			 SLOT( slotCopySelectedClicked( )));
}

//	filtersUpdated
//	==============

void MessagesView::filtersUpdated( )
{	m_controlView->filtersUpdated( );
}

//	showEvent
//	=========

void MessagesView::showEvent( QShowEvent* event )
{	QSplitter::showEvent( event );

	if( m_shown )
		return;

	m_listView->restoreLayout( kapp->config( ), "MessagesView_" + m_client->clientName );
	setSizes( m_sizes );
	m_shown = true;
}

//	saveSettings
//	============

void MessagesView::saveSettings( const bool clientSpecific )
{	m_controlView->saveSettings( );

	if( m_shown )
		m_listView->saveLayout( kapp->config( ), "MessagesView_" + m_client->clientName );

	KConfig* config = kapp->config( );

	if( clientSpecific )
		config->setGroup( "MessagesView_" + m_client->clientName );
	else
		config->setGroup( "MessagesView" );

	config->writeEntry( "viewWidths", m_shown ? sizes( ) : m_sizes );
}

//	slotAuthenticationFinished
//	==========================

void MessagesView::slotAuthenticationFinished( const CPtr client, const bool successful )
{	m_listView->authenticationFinished( client, successful );
}

//	slotAuthenticationStarted
//	=========================

void MessagesView::slotAuthenticationStarted( const CPtr client )
{	m_listView->authenticationStarted( client );
}

//	slotAutoProjectUpdateStarted
//	============================

void MessagesView::slotAutoProjectUpdateStarted( )
{	m_listView->autoProjectUpdateStarted( );
}

//	update
//	======

bool MessagesView::update( )
{	return m_listView->update( );
}
