/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>

#include <kdebug.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>

#include "gui_rpc_client.h"
#include "messagesviewitem.h"

//	constructor
//	===========

MessagesViewItem::MessagesViewItem( const byte     timeCol,
									QListView*     parent,
									const int      priority,
									const QString& projectName,
									const uint     timeStamp,
									const double   seqNo,
									const QString& text )
: KListViewItem( parent, projectName, QString::null, text ),
  m_colour( KGlobalSettings::textColor( )),
  m_priority( priority ),
  m_seqNo( seqNo ),
  m_timeCol( timeCol ),
  m_timeStamp( timeStamp),
  m_visible( true )
{	QDateTime dateTime;
	dateTime.setTime_t( m_timeStamp );
	setText( 1, KGlobal::locale( )->formatDateTime( dateTime, true, true ));
}

//	constructor
//	===========

MessagesViewItem::MessagesViewItem( MVIPtr         parent,
									const int      priority,
								    const double   seqNo,
								    const QString& text )
: KListViewItem( parent, "", "", text ),
  m_colour( KGlobalSettings::textColor( )),
  m_priority( priority ),
  m_seqNo( seqNo ),
  m_visible( true )
{
}

//	constructor
//	===========

MessagesViewItem::MessagesViewItem( MVIPtr         parent,
								    MVIPtr         sibling,
								    const int      priority,
								    const double   seqNo,
								    const QString& text )
: KListViewItem( parent, sibling, "", "", text ),
  m_colour( KGlobalSettings::textColor( )),
  m_priority( priority ),
  m_seqNo( seqNo ),
  m_visible( true )
{
}

//	compare
//	=======

int MessagesViewItem::compare( QListViewItem* otherItem, int col, bool ascending ) const
{	if( col != m_timeCol )
		return KListViewItem::compare( otherItem, col, ascending );

	int timeStampDiff = int( m_timeStamp ) - int( MVIPtr( otherItem )->m_timeStamp );

	if( timeStampDiff )
		return _sign_( timeStampDiff );

	double seqNoDiff = m_seqNo - MVIPtr( otherItem )->m_seqNo;

	if( seqNoDiff )
		return _sign_( seqNoDiff );

	Q_ASSERT( false );
	return 0;
}

//	paintCell
//	=========

void MessagesViewItem::paintCell( QPainter* painter, const QColorGroup& cg, int column, int width, int align )
{	QColorGroup _cg( cg );
	_cg.setColor( QColorGroup::Text, m_colour );
	KListViewItem::paintCell( painter, _cg, column, width, align );
}
