/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qcheckbox.h>
#include <qdatetimeedit.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qvgroupbox.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kfontdialog.h>
#include <klineedit.h>
#include <klocale.h>

#include "preferences.h"

//	constructor
//	===========

Preferences::Preferences( PrefData* prefData, QWidget* parent, const char* name )
: KDialogBase( Tabbed, i18n( "Preferences" ), Ok | Apply | Cancel, Ok, parent, name ),
  m_prefData( prefData )
{	prefData->fontUpdated = false;

	createRetryLayout( );
	createMiscLayout( );

	KConfig* config = kapp->config( );
	config->setGroup( "General" );
	showPage( config->readNumEntry( "preferencesActivePageIndex", 0 ));

	resize( 300, 500 );
}

//	createMiscLayout
//	================

void Preferences::createMiscLayout( )
{	QFrame* page = addPage( i18n( "Miscellaneous" ));
	QGridLayout* layout = new QGridLayout( page );
	byte row = 0;

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Update interval" ), page, "interval label" ), row++, 1 );
	m_updateSpinBox = new QSpinBox( 1, 0x7fffffff, 1, page, "interval spin box" );
	layout->addWidget( m_updateSpinBox, row++, 1 );
	m_updateSpinBox->setSuffix( i18n( " secs" ));
	m_updateSpinBox->setValue( m_prefData->updateInterval / 1000 );

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Project update interval" ), page, "project interval label" ), row++, 1 );
	m_projectSpinBox = new QSpinBox( 0, 0x7fffffff / 3600, 1, page, "interval spin box" );
	layout->addWidget( m_projectSpinBox, row++, 1 );
	m_projectSpinBox->setSpecialValueText( i18n( "never" ));
	m_projectSpinBox->setSuffix( i18n( " days" ));
	m_projectSpinBox->setValue( m_prefData->projectUpdateInterval );

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Project update TOD" ), page, "project update tod label" ), row++, 1 );
	QTime projectUpdateTOD( m_prefData->projectUpdateTOD / 3600, (m_prefData->projectUpdateTOD / 60) % 60, m_prefData->projectUpdateTOD % 60 );
	m_projectUpdateTODEdit = new QTimeEdit( projectUpdateTOD, page, "project update tod edit" );
	layout->addWidget( m_projectUpdateTODEdit, row++, 1 );

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Wallet name" ), page, "wallet name label" ), row++, 1 );
	m_walletNameEdit = new KLineEdit( page, "wallet name edit" );
	layout->addWidget( m_walletNameEdit, row++, 1 );
	m_walletNameEdit->setText( m_prefData->walletName );

	layout->setRowStretch( row++, 10 );

	m_synchroniseLayouts = new QCheckBox( i18n( "Synchronise layouts" ), page, "synchronise layouts" );
	layout->addWidget( m_synchroniseLayouts, row++, 1 );
	m_synchroniseLayouts->setChecked( m_prefData->synchroniseLayouts );

	layout->setRowStretch( row++, 10 );

	QPushButton* fontButton = new QPushButton( i18n( "Font" ), page, "font button" );
	layout->addWidget( fontButton, row++, 1 );
	layout->setRowStretch( row++, 10 );

	connect( fontButton, SIGNAL( clicked( )), SLOT( slotFontClicked( )));

	layout->setColStretch( 0,  2 );
	layout->setColStretch( 1, 10 );
	layout->setColStretch( 2,  2 );
}

//	createRetryLayout
//	=================

void Preferences::createRetryLayout( )
{	QFrame* page = addPage( i18n( "Retry control" ));
	QGridLayout* layout = new QGridLayout( page );
	byte row = 0;

	layout->setRowStretch( row++, 10 );

	m_retryCheckBox = new QCheckBox( i18n( "Retry lost connections" ), page, "retry check box" );
	m_retryCheckBox->setChecked( m_prefData->retryLostCxns );
	layout->addWidget( m_retryCheckBox, row++, 1 );
	connect( m_retryCheckBox, SIGNAL( toggled( bool )), SLOT( slotRetryToggled( bool )));

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Retry limit" ), page, "retry limit label" ), row++, 1 );
	m_retryLimitSpinBox = new QSpinBox( -1, 0x7fffffff, 1, page, "retry limit spin box" );
	layout->addWidget( m_retryLimitSpinBox, row++, 1 );
	m_retryLimitSpinBox->setValue( m_prefData->retryLimit );
	m_retryLimitSpinBox->setSpecialValueText( "Unlimited" );
	m_retryLimitSpinBox->setEnabled( m_prefData->retryLostCxns );

	layout->setRowStretch( row++, 10 );

	m_retryExpBackCheckBox = new QCheckBox( i18n( "Exponential backoff" ), page, "exp back check box" );
	layout->addWidget( m_retryExpBackCheckBox, row++, 1 );
	m_retryExpBackCheckBox->setChecked( m_prefData->retryExpBack );
	m_retryExpBackCheckBox->setEnabled( m_prefData->retryLostCxns );
	connect( m_retryExpBackCheckBox, SIGNAL( toggled( bool )), SLOT( slotExpBackToggled( bool )));

	layout->setRowStretch( row++, 10 );

	layout->addWidget( new QLabel( i18n( "Exponential backoff limit" ), page, "exp back limit label" ), row++, 1 );
	m_retryExpBackSpinBox = new QSpinBox( 1, 0x7fffffff, 10, page, "exp back limit spin box" );
	layout->addWidget( m_retryExpBackSpinBox, row++, 1 );
	m_retryExpBackSpinBox->setValue( m_prefData->retryExpBackLimit );
	m_retryExpBackSpinBox->setEnabled( m_prefData->retryLostCxns && m_prefData->retryExpBack );
	m_retryExpBackSpinBox->setSuffix( " secs" );

	layout->setRowStretch( row++, 80 );

	layout->setColStretch( 0,  2 );
	layout->setColStretch( 1, 10 );
	layout->setColStretch( 2,  2 );
}

//	slotApply
//	=========

void Preferences::slotApply( )
{	m_prefData->projectUpdateInterval = m_projectSpinBox->value( );
	m_prefData->retryExpBackLimit     = m_retryExpBackSpinBox->value( );
	m_prefData->retryLimit            = m_retryLimitSpinBox->value( );
	m_prefData->synchroniseLayouts    = m_synchroniseLayouts->isChecked( );
	m_prefData->updateInterval        = 1000 * m_updateSpinBox->value( );
	m_prefData->walletName            = m_walletNameEdit->text( );

	QTime projectUpdateTOD            = m_projectUpdateTODEdit->time( );
	m_prefData->projectUpdateTOD      = 3600 * projectUpdateTOD.hour( )   +
										  60 * projectUpdateTOD.minute( ) +
											   projectUpdateTOD.second( );

	emit( applyPreferences( m_prefData ));
}

//	slotExpBackToggled
//	==================

void Preferences::slotExpBackToggled( bool on )
{	m_prefData->retryExpBack = on;
	m_retryExpBackSpinBox->setEnabled( on );
}

//	slotFontClicked
//	===============

void Preferences::slotFontClicked( )
{	if( KFontDialog::getFont( m_prefData->font ) == QDialog::Accepted )
		m_prefData->fontUpdated = true;
}

//	slotOk
//	======

void Preferences::slotOk( )
{	KConfig* config = kapp->config( );
	config->setGroup( "General" );
	config->writeEntry( "preferencesActivePageIndex", activePageIndex( ));

	slotApply( );
	KDialogBase::slotOk( );
	delete m_prefData;
}

//	slotRetryToggled
//	================

void Preferences::slotRetryToggled( bool on )
{	m_prefData->retryLostCxns = on;

	m_retryExpBackCheckBox->setEnabled( on );
	m_retryExpBackSpinBox->setEnabled( on && m_prefData->retryExpBack );
	m_retryLimitSpinBox->setEnabled( on );
}
