/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _PREFERENCES_H_
#define _PREFERENCES_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qdatetime.h>
#include <qfont.h>
#include <kdialogbase.h>
#include "global.h"

//	forward class definitions
//	=========================

class KLineEdit;
class QCheckBox;
class QSpinBox;
class QTimeEdit;

//	Preferences class definition
//	============================

class Preferences : public KDialogBase
{	Q_OBJECT

	public:
		typedef struct
		{	QFont   font;
			bool    fontUpdated;
			uint    projectUpdateInterval;
			uint    projectUpdateTOD;
			bool    retryExpBack;
			uint    retryExpBackLimit;
			int     retryLimit;
			bool    retryLostCxns;
			bool    synchroniseLayouts;
			uint    updateInterval;
			QString walletName;
		} PrefData;

		Preferences( PrefData* prefData, QWidget* parent, const char* name );

	signals:
		void applyPreferences( const Preferences::PrefData* );

	private slots:
		void slotExpBackToggled( bool on );
		void slotFontClicked( );
		void slotRetryToggled( bool on );

	private:
		void createMiscLayout( );
		void createRetryLayout( );
		void slotApply( );
		void slotOk( );

		PrefData*  m_prefData;
		QSpinBox*  m_projectSpinBox;
		QTimeEdit* m_projectUpdateTODEdit;
		QCheckBox* m_retryExpBackCheckBox;
		QSpinBox*  m_retryExpBackSpinBox;
		QSpinBox*  m_retryLimitSpinBox;
		QCheckBox* m_retryCheckBox;
		QCheckBox* m_synchroniseLayouts;
		QSpinBox*  m_updateSpinBox;
		KLineEdit* m_walletNameEdit;
};

#endif	// _PREFERENCES_H_
