/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qpushbutton.h>
#include <qvbox.h>

#include <kglobalsettings.h>

#include "projectscontrolview.h"

//	constructor
//	===========

ProjectsControlView::ProjectsControlView( QWidget* parent, const char* name )
: QScrollView( parent, name )
{	byte spacing = fontMetrics( ).width( "XX" );

	QVBox* vBox = new QVBox( viewport( ));
	vBox->setEraseColor( viewport( )->eraseColor( ));
	vBox->setMargin( spacing / 2 );
	vBox->setSpacing( spacing );

	addChild( vBox );

	m_update         = new QPushButton( "Update",          vBox, "update button"         );
	m_suspend        = true;
	QWidget* spacer1 = new QWidget( vBox );
	m_suspendResume  = new QPushButton( "Suspend project", vBox, "suspen /resume button" );
	QWidget* spacer2 = new QWidget( vBox );
	m_reset          = new QPushButton( "Reset project",   vBox, "reset project button"  );
	m_detach         = new QPushButton( "Detach",          vBox, "detach button"         );

	spacer1->setEraseColor( KGlobalSettings::buttonBackground( ));
	spacer2->setEraseColor( KGlobalSettings::buttonBackground( ));

	vBox->setStretchFactor( spacer1, 10 );
	vBox->setStretchFactor( spacer2, 10 );

	m_detach->setEnabled( false );
	m_reset->setEnabled( false );
	m_suspendResume->setEnabled( false );
	m_update->setEnabled( false );

	connect( m_detach,        SIGNAL( clicked( )), SIGNAL( detachClicked( )));
	connect( m_reset,         SIGNAL( clicked( )), SIGNAL( resetClicked( )));
	connect( m_suspendResume, SIGNAL( clicked( )), SLOT( slotSuspendResume( )));
	connect( m_update,        SIGNAL( clicked( )), SIGNAL( updateClicked( )));
}

//	slotEnableButtons
//	=================

void ProjectsControlView::slotEnableButtons( const bool b, const bool suspended )
{	m_detach->setEnabled( b );
	m_reset->setEnabled( b );
	m_suspendResume->setEnabled( b );
	m_update->setEnabled( b );

	slotProjectSuspended( suspended );
}

//	slotProjectSuspended
//	====================

void ProjectsControlView::slotProjectSuspended( const bool suspended )
{	m_suspend = suspended;
	m_suspendResume->setText( m_suspend ? "Resume project" : "Suspend project" );
}

//	slotSuspendResume
//	=================

void ProjectsControlView::slotSuspendResume( )
{	emit( suspendResumeClicked( m_suspend ));
	m_suspend = !m_suspend;
	m_suspendResume->setText( m_suspend ? "Suspend project" : "Resume project" );
}
