/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _PROJECTSLISTVIEW_H_
#define _PROJECTSLISTVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qptrlist.h>
#include <klistview.h>
#include "global.h"
#include "gui_rpc_client.h"

//	forward class definitions
//	=========================

class PROJECT;
class ProjectsViewItem;
class RPC_CLIENT;

//	ProjectsListView class definition
//	=============================

class ProjectsListView : public KListView
{	Q_OBJECT

	public:
		ProjectsListView( const CPtr client, QWidget* parent, const char* name );
		bool update( );
		void updateProjects( );

	signals:
		void autoProjectUpdateStarted( );
		void currentProject( const QString& );
		void enableButtons( const bool, const bool );
		void guiURLs( const PROJECT* );
		void projectSuspended( const bool );

	public slots:
		void slotDetachClicked( );
		void slotResetClicked( );
		void slotSuspendResume( const bool suspended );
		void slotUpdateClicked( );

	private slots:
		void slotAuthenticationFinished( const CPtr client, const bool successful );
		void slotSelectionChanged( );

	private:
		enum Operation
		{	Detach,
			Reset,
			Resume,
			Suspend,
			Update
		};

		void doProjectOp( const PVIPtr item, const Operation op );
		void doProjectUpdates( );
		void processNewProjects( QPtrList<PROJECT> newProjectList, const double totalResourceShare );
		void update( PVIPtr item, PROJECT* project, const double totalResourceShare );

		CPtr m_client;
		bool m_firstProject;
};

#endif	// _PROJECTSLISTVIEW_H_
