/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qwidgetstack.h>

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "projectscontrolview.h"
#include "projectslistview.h"
#include "projectsview.h"

//	constructor
//	===========

ProjectsView::ProjectsView( const CPtr client, QWidget* parent, const char* name )
: QSplitter( parent, name ),
  m_client( client ),
  m_shown( false )
{	m_controlView    = new ProjectsControlView( this, "projects control view" );
	m_guiURLSplitter = new QSplitter( this, "gui url splitter" );
	m_listView       = new ProjectsListView( m_client, m_guiURLSplitter, "projects list view" );
	m_guiURLStack    = new QWidgetStack( m_guiURLSplitter, "gui url stack" );

	KConfig* config = kapp->config( );
	config->setGroup( "MainWindow" );
	bool synchroniseLayouts = config->readBoolEntry( "synchroniseLayouts", false );

	if( synchroniseLayouts )
		config->setGroup( "ProjectsView" );
	else
		config->setGroup( "ProjectsView_" + m_client->clientName );

	m_guiURLSizes = config->readIntListEntry( "guiURLSplitter" );
	m_sizes       = config->readIntListEntry( "viewWidths"     );

	connect( m_controlView,
			 SIGNAL( detachClicked( )),
			 m_listView,
			 SLOT( slotDetachClicked( )));

	connect( m_controlView,
			 SIGNAL( resetClicked( )),
			 m_listView,
			 SLOT( slotResetClicked( )));

	connect( m_controlView,
			 SIGNAL( suspendResumeClicked( const bool )),
			 m_listView,
			 SLOT( slotSuspendResume( const bool )));

	connect( m_controlView,
			 SIGNAL( updateClicked( )),
			 m_listView,
			 SLOT( slotUpdateClicked( )));

	connect( m_listView,
			 SIGNAL( autoProjectUpdateStarted( )),
			 SIGNAL( autoProjectUpdateStarted( )));

	connect( m_listView,
			 SIGNAL( currentProject( const QString& )),
			 SLOT( slotCurrentProject( const QString& )));

	connect( m_listView,
			 SIGNAL( enableButtons( const bool, const bool )),
			 m_controlView,
			 SLOT( slotEnableButtons( const bool, const bool )));

	connect( m_listView,
			 SIGNAL( guiURLs( const PROJECT* )),
			 SLOT( slotGuiURLs( const PROJECT* )));

	connect( m_listView,
			 SIGNAL( projectSuspended( const bool )),
			 m_controlView,
			 SLOT( slotProjectSuspended( const bool )));
}

//	saveSettings
//	============

void ProjectsView::saveSettings( const bool clientSpecific )
{	if( m_shown )
		m_listView->saveLayout( kapp->config( ), "ProjectsView_" + m_client->clientName );

	KConfig* config = kapp->config( );

	if( clientSpecific )
		config->setGroup( "ProjectsView_" + m_client->clientName );
	else
		config->setGroup( "ProjectsView" );

	config->writeEntry( "guiURLSplitter", m_shown ? m_guiURLSplitter->sizes( ) : m_guiURLSizes );
	config->writeEntry( "viewWidths",  m_shown ? sizes( ) : m_sizes );
}

//	showEvent
//	=========

void ProjectsView::showEvent( QShowEvent* event )
{	QSplitter::showEvent( event );

	if( m_shown )
		return;

	setSizes( m_sizes );
	m_guiURLSplitter->setSizes( m_guiURLSizes );
	m_listView->restoreLayout( kapp->config( ), "ProjectsView_" + m_client->clientName );

	m_shown = true;
}

//	slotCurrentProject
//	==================

void ProjectsView::slotCurrentProject( const QString& projectName )
{	for( GuiURLView* view = m_guiURLViews.first( ); view != NULL; view = m_guiURLViews.next( ))
		if( view->projectName( ) == projectName )
		{	m_guiURLStack->raiseWidget( view );
			return;
		}
}

//	slotGuiURLs
//	===========

void ProjectsView::slotGuiURLs( const PROJECT* project )
{	GuiURLView* guiURLView = new GuiURLView( project, m_guiURLStack, "gui url view" );
	m_guiURLViews.append( guiURLView );
	m_guiURLStack->addWidget( guiURLView );
}

//	slotUpdateProjects
//	==================

void ProjectsView::slotUpdateProjects( )
{	m_listView->updateProjects( );
}

//	update
//	======

bool ProjectsView::update( )
{	return m_listView->update( );
}
