/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _RESULTSLISTVIEW_H_
#define _RESULTSLISTVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <klistview.h>

#include "global.h"
#include "ptrlist.h"
#include "resultsviewitem.h"

//	forward class definitions
//	=========================

class RESULT;
class RPC_CLIENT;

//	ResultsView class definition
//	============================

class ResultsListView : public KListView
{	Q_OBJECT

	public:
		ResultsListView( const CPtr client, QWidget* parent, const char* name );

		QString toolTipText( );
		bool    update( );

	signals:
		void enableButtons( const bool, const bool, const bool, const bool );

	public slots:
		void slotAbortClicked( );
		void slotGraphicsHideClicked( );
		void slotGraphicsShowClicked( );
		void slotResumeClicked( );
		void slotSuspendClicked( );

	private slots:
		void slotSelectionChanged( );

	private:
		enum Operation
		{	Abort,
			Resume,
			Suspend
		};

		void abortResumeOrSuspendResult( const Operation op );
		void abortResumeOrSuspendResult( const Operation op, ResultsViewItem* item );
		void processNewResults( QPtrList<RESULT>& newResultsList );
		void showOrHideGraphics( const QString& projectURL, const QString& resultName, const int graphicsMode );
		void update( ResultsViewItem* item, RESULT* result );

		CPtr m_client;
		PtrList<ResultsViewItem> m_results;
};

#endif	// _RESULTSLISTVIEW_H_
