/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "resultscontrolview.h"
#include "resultslistview.h"
#include "resultsview.h"

//	constructor
//	===========

ResultsView::ResultsView( const CPtr client, QWidget* parent, const char* name )
: QSplitter( parent, name ),
  m_client( client ),
  m_shown( false )
{	m_controlView = new ResultsControlView( this, "results control view" );
	m_listView    = new ResultsListView( m_client, this, "results list view" );

	KConfig* config = kapp->config( );
	config->setGroup( "MainWindow" );
	bool synchroniseLayouts = config->readBoolEntry( "synchroniseLayouts", false );

	if( synchroniseLayouts )
		config->setGroup( "ResultsView" );
	else
		config->setGroup( "ResultsView_" + m_client->clientName );

	m_sizes = config->readIntListEntry( "viewWidths" );

	connect( m_controlView,
			 SIGNAL( abortClicked( )),
			 m_listView,
			 SLOT( slotAbortClicked( )));

	connect( m_controlView,
			 SIGNAL( graphicsHideClicked( )),
			 m_listView,
			 SLOT( slotGraphicsHideClicked( )));

	connect( m_controlView,
			 SIGNAL( graphicsShowClicked( )),
			 m_listView,
			 SLOT( slotGraphicsShowClicked( )));

	connect( m_controlView,
			 SIGNAL( resumeClicked( )),
			 m_listView,
			 SLOT( slotResumeClicked( )));

	connect( m_controlView,
			 SIGNAL( suspendClicked( )),
			 m_listView,
			 SLOT( slotSuspendClicked( )));

	connect( m_listView,
			 SIGNAL( enableButtons( const bool, const bool, const bool, const bool )),
			 m_controlView,
			 SLOT( slotEnableButtons( const bool, const bool, const bool, const bool )));
}

//	showEvent
//	=========

void ResultsView::showEvent( QShowEvent* event )
{	QSplitter::showEvent( event );

	if( m_shown )
		return;

	m_listView->restoreLayout( kapp->config( ), "ResultsView_" + m_client->clientName );
	setSizes( m_sizes );
	m_shown = true;
}

//	saveSettings
//	============

void ResultsView::saveSettings( const bool clientSpecific )
{	if( m_shown )
		m_listView->saveLayout( kapp->config( ), "ResultsView_" + m_client->clientName );

	KConfig* config = kapp->config( );

	if( clientSpecific )
		config->setGroup( "ResultsView_" + m_client->clientName );
	else
		config->setGroup( "ResultsView" );

	config->writeEntry( "viewWidths", m_shown ? sizes( ) : m_sizes );
}

//	toolTipText
//	===========

QString ResultsView::toolTipText( )
{	return m_listView->toolTipText( );
}

//	update
//	======

bool ResultsView::update( )
{	return m_listView->update( );
}
