/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>

#include "gui_rpc_client.h"
#include "mainwidgetview.h"
#include "resultsviewitem.h"

//	constructor
//	===========

ResultsViewItem::ResultsViewItem( const byte progressCol, QListView* parent, const RESULT* result )
: KListViewItem( parent ),
  m_finished( false ),
  m_name( result->name.c_str( )),
  m_preempted( false ),
  m_progressCol( progressCol ),
  m_reportDeadline( result->report_deadline ),
  m_running( false ),
  m_suspended( false ),
  m_uploading( false ),
  m_waiting( false )
{
}

//	compare
//	=======

int ResultsViewItem::compare( QListViewItem* otherItem, int col, bool ascending ) const
{	if( col != m_progressCol )
		return KListViewItem::compare( otherItem, col, ascending );

	if( m_progress == 100 && ((ResultsViewItem*) otherItem)->m_progress != 100 )
		return -1;

	if( m_progress != 100 && ((ResultsViewItem*) otherItem)->m_progress == 100 )
		return 1;

	if( m_progress == 0 && ((ResultsViewItem*) otherItem)->m_progress != 0 )
		return 1;

	if( m_progress != 0 && ((ResultsViewItem*) otherItem)->m_progress == 0 )
		return -1;

	if( m_progress != ((ResultsViewItem*) otherItem)->m_progress )
		return m_progress < ((ResultsViewItem*) otherItem)->m_progress ? -1 : 1;

	if( m_reportDeadline != ((ResultsViewItem*) otherItem)->m_reportDeadline )
		return m_reportDeadline < ((ResultsViewItem*) otherItem)->m_reportDeadline ? -1 : 1;

	if( m_project != ((ResultsViewItem*) otherItem)->m_project )
		return m_project < ((ResultsViewItem*) otherItem)->m_project ? -1 : 1;

	if( m_name < ((ResultsViewItem*) otherItem)->m_name )
		return -1;

	return m_name != ((ResultsViewItem*) otherItem)->m_name;
}

//	setProgress
//	===========

void ResultsViewItem::setProgress( const double progress )
{	m_progress = progress;

	if( m_progress == 0 )
		setText( m_progressCol, QString::null );
	else
	{	QString text;
		text.sprintf( "%.2f%%", m_progress );
		setText( m_progressCol, text );
	}
}

//	setProject
//	==========

void ResultsViewItem::setProject( const byte col, const QString& project )
{	m_project = project;
	setText( col, m_project );
}

//	setRemainingCPU
//	===============

void ResultsViewItem::setRemainingCPU( const byte col, const uint remainingCPU )
{	m_remainingCPU = remainingCPU;
	setText( col, MainWidgetView::secondsToString( m_remainingCPU ));

}
