/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _STATISTICSCONTROLVIEW_H_
#define _STATISTICSCONTROLVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qptrlist.h>
#include <qscrollview.h>

#include "global.h"
#include "statisticscontroldata.h"

//	forward class definitions
//	=========================

class KComboBox;
class QRadioButton;

//	StatisticsControlView class definition
//	======================================

class StatisticsControlView : public QScrollView
{	Q_OBJECT

	public:
		StatisticsControlView( const CPtr client, QWidget* parent, const char* name );

		GraphType graphType( ) { return findCurrentProject( )->graphType( );   }
		QString projectName( ) { return findCurrentProject( )->projectName( ); }
		QPtrList<StatisticsControlData>* projectsData( ) { return &m_projectsData; }
		void saveSettings( );

	signals:
		void hostAverageActivated( const QString& );
		void hostTotalActivated( const QString& );
		void userAverageActivated( const QString& );
		void userTotalActivated( const QString& );

	private slots:
		void slotHostAvgClicked( );
		void slotHostTotalClicked( );
		void slotProjectActivated( const QString& /* projectName */);
		void slotUserAvgClicked( );
		void slotUserTotalClicked( );

	private:
		static int compareProjects( const void* project1, const void* project2 );
		void   createLayout( const QString& currentProject );
		void   finaliseInitialisation( );
		SCDPtr findCurrentProject( );
		void   hostAveragelActivated( );
		void   hostTotalActivated( );
		void   userAverageActivated( );
		void   userTotalActivated( );

		CPtr          m_client;
		QRadioButton* m_hostAvg;
		QRadioButton* m_hostTotal;
		KComboBox*    m_projectsComboBox;
		QRadioButton* m_userAvg;
		QRadioButton* m_userTotal;

		QPtrList<StatisticsControlData> m_projectsData;
};

#endif	// _STATISTICSCONTROLVIEW_H_
