/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _STATISTICSDATA_H_
#define _STATISTICSDATA_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qptrlist.h>
#include <qrect.h>
#include <qstring.h>
#include <qvaluelist.h>

#include "global.h"

class StatisticsData
{	public:
		StatisticsData( const PROJECTS& projects,
						QPtrList<StatisticsControlData>* projectsData,
						const QString& clientName,
						const bool stacked,
						QValueList<uint>* removedDataPoints );
	   ~StatisticsData( );

		uint    date( const ushort projectNo, const ushort i ) const;
		bool    interpolated( const ushort projectNo, const ushort dateIndex ) const;
		uint    maxDate( ) const;
		uint    minDate( ) const;
		double  maxValue( const ushort projectNo, const GraphType graphType, const bool stacked ) const;
		double  minValue( const ushort projectNo, const GraphType graphType, const bool stacked ) const;
		ushort  noOfDataPoints( ) const { return m_noOfDataPoints; }
		QRect   point( const ushort projectNo, const ushort dateIndex, const GraphType graphType ) const;
		void    point( const ushort projectNo, const ushort dateIndex, const GraphType graphType, const QRect& rect );
		bool    sameData( const PROJECTS& projects ) const;
		double  scaledValue( const ushort projectNo, const ushort dateIndex, const GraphType graphType ) const;
		QString toolTipText( const ushort projectNo, const ushort dateIndex, const GraphType graphType ) const;
		bool    valid( const ushort projectNo, const ushort i ) const;
		double  value( const ushort projectNo, const ushort dateIndex, const GraphType graphType ) const;

	private:

		void       buildToolTipText( QPtrList<StatisticsControlData>* projectsData, const QString& clientName );
		void       calcMinMax( );
		void       calcTotals( );
		static int compareDates( const void* date1, const void* date2 );
		void       createDataStructures( const PROJECTS& projects );
		void       deleteProjectData( );
		void       extractData( const PROJECTS& projects );
		void       scaleValues( const bool stacked );
		void       smoothStacked( const bool stacked );

		ushort m_noOfDataPoints;
		ushort m_noOfProjects;
		SPDPtr m_projectData;
		double m_totalHostAvg;
		double m_totalHostTotal;
		ushort m_totalProject;
		double m_totalUserAvg;
		double m_totalUserTotal;

		QValueList<uint>* m_removedDataPoints;
};

#endif	// _STATISTICSDATA_H_
