/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef _STATISTICSGRAPHVIEW_H_
#define _STATISTICSGRAPHVIEW_H_

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <qrect.h>
#include <qvaluelist.h>
#include <qwidget.h>
#include "global.h"

//	forward class definitions
//	=========================

class QMouseEvent;
class QPainter;
class QPaintEvent;
class QRect;
class StatsToolTip;

//	StatisticsGraphView class definition
//	====================================

class StatisticsGraphView : public QWidget
{	Q_OBJECT

	public:
		StatisticsGraphView( const CPtr client, QWidget* parent, const char* name );
	   ~StatisticsGraphView( );

		void setInitialSh1t( QPtrList<StatisticsControlData>* projectsData,
							 const GraphType graphType,
							 const QString&  projectName );
		bool update( const bool forceUpdate = false );

	public slots:
		void slotHostAverage( const QString& projectName );
		void slotHostTotal( const QString& projectName );
		void slotUserAverage( const QString& projectName );
		void slotUserTotal( const QString& projectName );

	private slots:
		void slotRemoveDataPoint( );

	private:
		typedef struct
		{	QString label1;
			QString label2;
		} Labels, *LPtr;

		void   buildLabels( QPainter* painter );
		void   createAndDrawPointRect( const QPoint& point, const ushort guiRPCIndex, const ushort i, QPainter* painter );
		void   drawGrid( QPainter* painter );
		void   drawNoData( QPainter* painter );
		void   drawProjectPlot( const ushort guiRPCIndex, QPainter* painter );
		ushort drawTitle( QPainter* painter, const QString& title );
		void   findProjectData( );
		bool   findStatsData( const QPoint& pos, const ushort guiRPCIndex, ushort& i );
		void   mouseMoveEvent( QMouseEvent* event );
		bool   mouseMoveEvent( const QPoint& pos, const ushort guiRPCIndex );
		void   mouseReleaseEvent( QMouseEvent* event );
		bool   mouseReleaseEvent( QMouseEvent* event, const ushort guiRPCIndex );
		void   paintEvent( QPaintEvent* event );
		void   resizeEvent( QResizeEvent* event );

		const byte Columns;
		const byte Margin;
		const byte PointSize;
		const byte Rows;

		CPtr          m_client;
		ushort        m_dotPos;
		QRect         m_graphRect;
		GraphType     m_graphType;
		bool          m_isNewData;
		LPtr          m_labelsCredit;
		LPtr          m_labelsDate;
		ushort        m_noOfProjects;
		SCDPtr        m_projectData;
		QString       m_projectName;
		ushort        m_removeGUIRPCIndex;
		ushort        m_removeIndex;
		SDPtr         m_statsData;
		StatsToolTip* m_toolTip;
		ushort        m_xWidth;

		QPtrList<StatisticsControlData>* m_projectsData;
		QValueList<uint> m_removedDataPoints;
};

#endif	// _STATISTICSGRAPHVIEW_H_
