/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>

#include <kglobal.h>
#include <klocale.h>

#include "statisticsprojectdata.h"

//	destructor
//	==========

StatisticsProjectData::~StatisticsProjectData( )
{	delete [] m_projectDateData;
}

//	buildToolTipText
//	================

void StatisticsProjectData::buildToolTipText( const QString& projectName, const QString& clientName, const bool stacked )
{	QDateTime date;

	for( ushort i = 0; i < m_noOfDataPoints; i++ )
	{	PDDPtr projectDateData = m_projectDateData + i;
		date.setTime_t( projectDateData->m_date );

		projectDateData->m_toolTipHostAvg.sprintf( "Host: %s\nProject: %s\nDate: %s\nHost Avg: %.2f",
			clientName.ascii( ),
			projectName.ascii( ),
			KGlobal::locale( )->formatDateTime( date, true, true ).ascii( ),
			projectDateData->m_hostAvg );

		projectDateData->m_toolTipHostTotal.sprintf( "Host: %s\nProject: %s\nDate: %s\nHost Total: %.2f",
			clientName.ascii( ),
			projectName.ascii( ),
			KGlobal::locale( )->formatDateTime( date, true, true ).ascii( ),
			projectDateData->m_hostTotal );

		projectDateData->m_toolTipUserAvg.sprintf( "Host: %s\nProject: %s\nDate: %s\nUser Avg: %.2f",
			clientName.ascii( ),
			projectName.ascii( ),
			KGlobal::locale( )->formatDateTime( date, true, true ).ascii( ),
			projectDateData->m_userAvg );

		projectDateData->m_toolTipUserTotal.sprintf( "Host: %s\nProject: %s\nDate: %s\nUser Total: %.2f",
			clientName.ascii( ),
			projectName.ascii( ),
			KGlobal::locale( )->formatDateTime( date, true, true ).ascii( ),
			projectDateData->m_userTotal );
	}

	if( !stacked )
		return;

	QString text;

	for( ushort i = 0; i < m_noOfDataPoints; i++ )
	{	PDDPtr projectDateData = m_projectDateData + i;
		text.sprintf( "\nInterpolated: %s", projectDateData->m_interpolated ? "yes" : " no" );

		projectDateData->m_toolTipHostAvg   += text;
		projectDateData->m_toolTipHostTotal += text;
		projectDateData->m_toolTipUserAvg   += text;
		projectDateData->m_toolTipUserTotal += text;
	}
}

//	calcMax
//	=======

void StatisticsProjectData::calcMax( )
{	ushort i;

	for( i = 0; i < m_noOfDataPoints; i++ )
		if( m_projectDateData[ i ].m_valid )
			break;

	if( i >= m_noOfDataPoints )
	{	m_maxHostAvg   = 0;
		m_maxHostTotal = 0;
		m_maxUserAvg   = 0;
		m_maxUserTotal = 0;

		return;
	}

	m_maxHostAvg   = m_projectDateData[ i ].m_hostAvg;
	m_maxHostTotal = m_projectDateData[ i ].m_hostTotal;
	m_maxUserAvg   = m_projectDateData[ i ].m_userAvg;
	m_maxUserTotal = m_projectDateData[ i ].m_userTotal;

	for( i++; i < m_noOfDataPoints; i++ )
		if( m_projectDateData[ i ].m_valid )
		{	m_maxHostAvg   = _max_( m_maxHostAvg,   m_projectDateData[ i ].m_hostAvg   );
			m_maxHostTotal = _max_( m_maxHostTotal, m_projectDateData[ i ].m_hostTotal );
			m_maxUserAvg   = _max_( m_maxUserAvg,   m_projectDateData[ i ].m_userAvg   );
			m_maxUserTotal = _max_( m_maxUserTotal, m_projectDateData[ i ].m_userTotal );
		}

	m_maxHostAvg   = 10 * uint( (m_maxHostAvg   + 9.9999) / 10 );
	m_maxHostTotal = 10 * uint( (m_maxHostTotal + 9.9999) / 10 );
	m_maxUserAvg   = 10 * uint( (m_maxUserAvg   + 9.9999) / 10 );
	m_maxUserTotal = 10 * uint( (m_maxUserTotal + 9.9999) / 10 );
}

//	calcMin
//	=======

void StatisticsProjectData::calcMin( )
{	ushort i;

	for( i = 0; i < m_noOfDataPoints; i++ )
		if( m_projectDateData[ i ].m_valid )
			break;

	if( i >= m_noOfDataPoints )
	{	m_minHostAvg   = 0;
		m_minHostTotal = 0;
		m_minUserAvg   = 0;
		m_minUserTotal = 0;

		return;
	}

	m_minHostAvg   = m_projectDateData[ i ].m_hostAvg;
	m_minHostTotal = m_projectDateData[ i ].m_hostTotal;
	m_minUserAvg   = m_projectDateData[ i ].m_userAvg;
	m_minUserTotal = m_projectDateData[ i ].m_userTotal;

	for( i++; i < m_noOfDataPoints; i++ )
		if( m_projectDateData[ i ].m_valid )
		{	m_minHostAvg   = _min_( m_minHostAvg,   m_projectDateData[ i ].m_hostAvg   );
			m_minHostTotal = _min_( m_minHostTotal, m_projectDateData[ i ].m_hostTotal );
			m_minUserAvg   = _min_( m_minUserAvg,   m_projectDateData[ i ].m_userAvg   );
			m_minUserTotal = _min_( m_minUserTotal, m_projectDateData[ i ].m_userTotal );
		}

	m_minHostAvg   = 10 * uint( m_minHostAvg   / 10 );
	m_minHostTotal = 10 * uint( m_minHostTotal / 10 );
	m_minUserAvg   = 10 * uint( m_minUserAvg   / 10 );
	m_minUserTotal = 10 * uint( m_minUserTotal / 10 );
}

//	initialise
//	==========

void StatisticsProjectData::initialise( const ushort noOfDataPoints )
{	m_noOfDataPoints  = noOfDataPoints;
	m_projectDateData = new ProjectDateData[ noOfDataPoints ];

	memset( m_projectDateData, 0, noOfDataPoints * sizeof( ProjectDateData ));
}

//	scaleValues
//	===========

void StatisticsProjectData::scaleValues( const StatisticsProjectData& minMaxProject )
{	double diffHostAvg   = minMaxProject.m_maxHostAvg   - minMaxProject.m_minHostAvg;
	double diffHostTotal = minMaxProject.m_maxHostTotal - minMaxProject.m_minHostTotal;
	double diffUserAvg   = minMaxProject.m_maxUserAvg   - minMaxProject.m_minUserAvg;
	double diffUserTotal = minMaxProject.m_maxUserTotal - minMaxProject.m_minUserTotal;

	for( ushort i = 0; i < m_noOfDataPoints; i++ )
	{	PDDPtr projectDateData = m_projectDateData + i;

		if( projectDateData->m_valid )
		{	projectDateData->m_scaledHostAvg   = (projectDateData->m_hostAvg   - minMaxProject.m_minHostAvg)   / diffHostAvg;
			projectDateData->m_scaledHostTotal = (projectDateData->m_hostTotal - minMaxProject.m_minHostTotal) / diffHostTotal;
			projectDateData->m_scaledUserAvg   = (projectDateData->m_userAvg   - minMaxProject.m_minUserAvg)   / diffUserAvg;
			projectDateData->m_scaledUserTotal = (projectDateData->m_userTotal - minMaxProject.m_minUserTotal) / diffUserTotal;
		}
	}
}
