/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "statisticscontrolview.h"
#include "statisticsgraphview.h"
#include "statisticsview.h"

//	constructor
//	===========

StatisticsView::StatisticsView( const CPtr client, QWidget* parent, const char* name )
: QSplitter( parent, name ),
  m_client( client ),
  m_shown( false )
{	m_controlView = new StatisticsControlView( m_client, this, "statistics control view" );
	m_graphView   = new StatisticsGraphView( m_client, this, "statistics graph view" );

	m_graphView->setInitialSh1t( m_controlView->projectsData( ), m_controlView->graphType( ), m_controlView->projectName( ));

	KConfig* config = kapp->config( );
	config->setGroup( "MainWindow" );
	bool synchroniseLayouts = config->readBoolEntry( "synchroniseLayouts", false );

	if( synchroniseLayouts )
		config->setGroup( "StatisticsView" );
	else
		config->setGroup( "StatisticsView_" + m_client->clientName );

	m_sizes = config->readIntListEntry( "viewWidths" );

	connect( m_controlView,
			 SIGNAL( hostAverageActivated( const QString& )),
			 m_graphView,
			 SLOT( slotHostAverage( const QString& )));
	connect( m_controlView,
			 SIGNAL( hostTotalActivated( const QString& )),
			 m_graphView,
			 SLOT( slotHostTotal( const QString& )));
	connect( m_controlView,
			 SIGNAL( userAverageActivated( const QString& )),
			 m_graphView,
			 SLOT( slotUserAverage( const QString& )));
	connect( m_controlView,
			 SIGNAL( userTotalActivated( const QString& )),
			 m_graphView,
			 SLOT( slotUserTotal( const QString& )));
}

//	showEvent
//	=========

void StatisticsView::showEvent( QShowEvent* event )
{	QSplitter::showEvent( event );

	if( m_shown )
		return;

	setSizes( m_sizes );
	m_shown = true;
}

//	saveSettings
//	============

void StatisticsView::saveSettings( const bool clientSpecific )
{	m_controlView->saveSettings( );
	KConfig* config = kapp->config( );

	if( clientSpecific )
		config->setGroup( "StatisticsView_" + m_client->clientName );
	else
		config->setGroup( "StatisticsView" );

	config->writeEntry( "viewWidths", m_shown ? sizes( ) : m_sizes );
}

//	update
//	======

bool StatisticsView::update( )
{	return m_graphView->update( );
}
