/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qvbox.h>

#include <kdebug.h>
#include <kglobalsettings.h>
#include <klocale.h>

#include "authenticate.h"
#include "transferscontrolview.h"

//	constructor
//	===========

TransfersControlView::TransfersControlView( const CPtr client, QWidget* parent, const char* name )
: QScrollView( parent, name ),
  m_client( client )
{	byte spacing = fontMetrics( ).width( "XX" );

	QVBox* vBox = new QVBox( viewport( ));
	vBox->setEraseColor( viewport( )->eraseColor( ));
	vBox->setMargin( spacing / 2 );
	vBox->setSpacing( spacing );

	addChild( vBox );

	QWidget* spacer1 = new QWidget( vBox );
	spacer1->setEraseColor( KGlobalSettings::buttonBackground( ));

	m_retry  = new QPushButton( i18n( "Retry now"         ), vBox, "retry button"  );

	QWidget* spacer2 = new QWidget( vBox );
	spacer2->setEraseColor( KGlobalSettings::buttonBackground( ));
	vBox->setStretchFactor( spacer2, 10 );

	m_auth   = new QPushButton( i18n( "Authenticate"      ), vBox, "auth button"   );
	m_config = new QPushButton( i18n( "Configure "        ), vBox, "config button" );

	QWidget* spacer3 = new QWidget( vBox );
	spacer3->setEraseColor( KGlobalSettings::buttonBackground( ));
	vBox->setStretchFactor( spacer3, 10 );

	m_abort  = new QPushButton( i18n( "Abort transfer(s)" ), vBox, "abort button"  );

	m_abort->setEnabled( false );
	m_auth->setEnabled( m_client->authenticate->authReqd( ));
	m_retry->setEnabled( false );

	connect( m_abort,  SIGNAL( clicked( )), SIGNAL( abortClicked( )));
	connect( m_retry,  SIGNAL( clicked( )), SIGNAL( retryClicked( )));

	connect( m_auth,   SIGNAL( clicked( )), SLOT( slotAuthClicked( )));
	connect( m_config, SIGNAL( clicked( )), SLOT( slotConfigClicked( )));
}

//	disableAuthButton
//	=================

void TransfersControlView::disableAuthButton( )
{	m_auth->setEnabled( false );
}

//	enableAuthButton
//	=================

void TransfersControlView::enableAuthButton( )
{	m_auth->setEnabled( true );
}

//	slotAuthClicked
//	===============

void TransfersControlView::slotAuthClicked( )
{	m_client->authenticate->authenticate( );
}

//	slotConfigClicked
//	=================

void TransfersControlView::slotConfigClicked( )
{	m_client->authenticate->configure( );
}

//	slotEnableButtons
//	=================

void TransfersControlView::slotEnableButtons( const bool enableAbort, const bool enableRetry )
{	m_abort->setEnabled( enableAbort );
	m_retry->setEnabled( enableRetry );
}
