/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>

#include "transferscontrolview.h"
#include "transferslistview.h"
#include "transfersview.h"

//	constructor
//	===========

TransfersView::TransfersView( const CPtr client, QWidget* parent, const char* name )
: QSplitter( parent, name ),
  m_client( client ),
  m_shown( false )
{	m_controlView = new TransfersControlView( m_client, this, "transfers control view" );
	m_listView    = new TransfersListView( m_client, this, "transfers list view" );

	KConfig* config = kapp->config( );
	config->setGroup( "MainWindow" );
	bool synchroniseLayouts = config->readBoolEntry( "synchroniseLayouts", false );

	if( synchroniseLayouts )
		config->setGroup( "TransfersView" );
	else
		config->setGroup( "TransfersView_" + m_client->clientName );

	m_sizes = config->readIntListEntry( "viewWidths" );

	connect( m_controlView,
			 SIGNAL( abortClicked( )),
			 m_listView,
			 SLOT( slotAbortClicked( )));

	connect( m_controlView,
			 SIGNAL( retryClicked( )),
			 m_listView,
			 SLOT( slotRetryClicked( )));

	connect( m_listView,
			 SIGNAL( enableButtons( const bool, const bool )),
			 m_controlView,
			 SLOT( slotEnableButtons( const bool, const bool )));
}

//	saveSettings
//	============

void TransfersView::saveSettings( const bool clientSpecific )
{	if( m_shown )
		m_listView->saveLayout( kapp->config( ), "TransfersView_" + m_client->clientName );

	KConfig* config = kapp->config( );

	if( clientSpecific )
		config->setGroup( "TransfersView_" + m_client->clientName );
	else
		config->setGroup( "TransfersView" );

	config->writeEntry( "viewWidths", m_shown ? sizes( ) : m_sizes );
}

//	showEvent
//	=========

void TransfersView::showEvent( QShowEvent* event )
{	QSplitter::showEvent( event );

	if( m_shown )
		return;

	m_listView->restoreLayout( kapp->config( ), "TransfersView_" + m_client->clientName );
	setSizes( m_sizes );
	m_shown = true;
}

//	slotAuthenticationFinished
//	==========================

void TransfersView::slotAuthenticationFinished( const CPtr /*client */, const bool successful)
{	m_listView->authenticationFinished( m_client, successful );
}

//	slotDisableAuthButton
//	=====================

void TransfersView::slotDisableAuthButton( )
{	m_controlView->disableAuthButton( );
}

//	slotEnableAuthButton
//	====================

void TransfersView::slotEnableAuthButton( )
{	m_controlView->enableAuthButton( );
}

//	update
//	======

bool TransfersView::update( )
{	return m_listView->update( );
}
